'''
Plotting libray, using Qt4 for expEYES
Author  : Georges Khaznadar <georgesk@debian.org>
Based on Ajith Kumar's work
License : GNU GPL version 3
'''

from PyQt4.QtCore import *
from PyQt4.QtGui import *
import time

class axisWidget(QGraphicsView):
    def __init__(self, parent=None, min = -5.0, max = 5.0, label = ''):
        """
        constructor
        @param parent the parent widget
        @param min minimum value
        @param max maximum value
        @param labels text to display as a label
        """
        QGraphicsView.__init__(self, parent)
        self.parent = parent
        self.min=min
        self.max=max
        self.label = label
        self.axis=[]
        self.labelFont=QFont("Courier New")
        self.labelFont.setPixelSize(11)
        self.textPen = QPen(Qt.blue)
        self.labelPen = QPen(Qt.black)
        self.numdiv = 5
        self.t=None # a dummy text to enforce redrawing

    def direction(self):
        """
        @return either 'horizontal' or 'vertical'
        """
        if not self.scene():
            return None
        if self.scene().sceneRect().width() > self.scene().sceneRect().height():
            return 'horizontal'
        else:
            return 'vertical'
    
    def setGeometry(self, rect):
        """
        redefinition of the default setGeometry
        @param rect a QRect
        """
        QGraphicsView.setGeometry(self, rect)
        scene = QGraphicsScene(QRectF(rect), self.parent)
        w, h = rect.width(), rect.height()
        margin = 4
        w-=margin
        h-=margin
        scene.setSceneRect(QRectF(0,0,w,h))
        self.setScene(scene)
        self.drawAxis()

    def setRange(self, min, max, label=''):
        """
        set a new range and label
        @param min minimum value
        @param max maximum value
        @param label a short text
        """
        self.min=min
        self.max=max
        self.label=label
        self.drawAxis()

    def drawAxis(self):
        """
        draws the axis
        """
        self.axis=[]
        if self.direction() == 'horizontal':
            pos=QPoint(0.9*self.width(), 1)
            self.axis.append((self.textPen, pos, self.labelFont, "(%s)"%self.label, "center", "bottom" ))
            dx = float(self.width())/self.numdiv
            align=["right"]+["center"]*(self.numdiv-1)+["left"]
            for x in range(0,self.numdiv+1):
                a = x *(self.max - self.min)/self.numdiv + self.min
                s = '%4.1f'%(a)
                pos=QPoint(x*dx, 1)
                self.axis.append((self.textPen, pos, self.labelFont, s, align[x], "bottom"))
        else: #position != 'horizontal'
            pos=QPoint(2,0.1*self.height())
            self.axis.append((self.textPen, pos, self.labelFont, "(%s)" %self.label, "right", "top"))
            dy = float(self.height())/self.numdiv
            align=["bottom"]+["center"]*(self.numdiv-1)+["top"]
            for y in range(0,self.numdiv+1):
                a = y*(self.max - self.min)/5
                if self.max > 99:
                    s = '%4.0f'%(self.max-a)
                else:
                    s = '%4.1f'%(self.max-a)
                pos=QPoint(self.width(), int(y*dy))
                self.axis.append((self.textPen, pos, self.labelFont, s, "left", align[y]))
        # enforce a repaint
        self.scene().invalidate(self.sceneRect())

    def paintEvent(self, event=None):
        """
        redefinition of the SLOT which deals with paint events (raised
        by repaint or update)
        @param event the paint event
        """
        QGraphicsView.paintEvent(self, event)
        painter=QPainter(self.viewport())
        painter.save()
        for pen, pos, font, text, xlayout, ylayout in self.axis:
            painter.setPen(pen)
            painter.setFont(font)
            fm=painter.fontMetrics()
            w=fm.width(text)+5
            h=fm.height()
            x=pos.x()
            y=pos.y()
            if xlayout == "left":
                x-=w
            elif xlayout == "center":
                x-=w/2
            if ylayout == "bottom":
                y+=h
            elif ylayout == "top":
                y-=h/2
            pos=QPoint(x,y)
            painter.drawText(pos, text)
        painter.restore()



# dependencies: python3-pyqt5.qtopengl python3-pyqt5.qtsvg
  
from PyQt5.QtCore import QFile, QSize, Qt
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtSvg import QGraphicsSvgItem

class MainWindow(QMainWindow):
    def __init__(self):
        super(MainWindow, self).__init__()
  
        self.currentFile = ''
  
        self.svg = SvgView()
  
        fileMenu = QMenu("&File", self)
        openAction = fileMenu.addAction("&Open...")
        openAction.setShortcut("Ctrl+O")
        quitAction = fileMenu.addAction("E&xit")
        quitAction.setShortcut("Ctrl+Q")
  
        self.menuBar().addMenu(fileMenu)
  
  
        openAction.triggered.connect(self.openFile)
        quitAction.triggered.connect(QApplication.instance().quit)
  
        self.setCentralWidget(self.svg)
        self.setWindowTitle("SVG Viewer")
  
    def openFile(self, path=None):
        if not path:
            path, _ = QFileDialog.getOpenFileName(self, "Open SVG File",
                    self.currentFile, "SVG files (*.svg *.svgz *.svg.gz)")
        if path:
            svg_file = QFile(path)
            if not svg_file.exists():
                QMessageBox.critical(self, "Open SVG File",
                        "Could not open file '%s'." % path)
  
                return
  
            self.svg.openFile(svg_file)
  
            self.currentFile = path
            self.setWindowTitle("%s - SVGViewer" % self.currentFile)
  
            self.resize(self.svg.sizeHint() + QSize(80, 80 + self.menuBar().height()))
  
class SvgView(QGraphicsView):
    def __init__(self, parent=None):
        super(SvgView, self).__init__(parent)
  
        self.setViewport(QWidget())
        self.svgItem = None
        self.image = QImage()
  
        self.setScene(QGraphicsScene(self))
        self.setTransformationAnchor(QGraphicsView.AnchorUnderMouse)
        self.setDragMode(QGraphicsView.ScrollHandDrag)
        self.setViewportUpdateMode(QGraphicsView.FullViewportUpdate)
  
    def openFile(self, svg_file):
        if not svg_file.exists():
            return
  
        s = self.scene()
    
        s.clear()
        self.resetTransform()
  
        self.svgItem = QGraphicsSvgItem(svg_file.fileName())
        self.svgItem.setFlags(QGraphicsItem.ItemClipsToShape)
        self.svgItem.setCacheMode(QGraphicsItem.NoCache)
        self.svgItem.setZValue(0)
    
        s.addItem(self.svgItem)

        self.printHotRectangles()

    def printHotRectangles(self):
        names="GND0,GND1,GND2,GND3,A1,A2,PVS,OUT,IN,MIC,SINE,IN1,IN2,SEN,SQR1,SQR2,OD1,CCS".split(",")
        for n in names:
            print(n, self.svgItem.renderer().boundsOnElement(n))

        
    def wheelEvent(self, event):
        factor = pow(1.2, event.angleDelta().y() / 240.0)
        self.scale(factor, factor)
        event.accept()

if __name__ == '__main__':
  
    import sys
  
    app = QApplication(sys.argv)
  
    window = MainWindow()
    window.openFile('ej1.svg')
    window.show()
    sys.exit(app.exec_())



import time, math, sys
if sys.version_info.major==3:
        from tkinter import *
else:
        from Tkinter import *

sys.path=[".."] + sys.path

import expeyes.eyesj as eyes
import expeyes.eyeplot as eyeplot
import expeyes.eyemath as eyemath

p=eyes.open()

import gettext
gettext.bindtextdomain("expeyes")
gettext.textdomain('expeyes')
_ = gettext.gettext

def set_freq(w):
	state = int(Sqr1.get())
	fr = float(Sqr1.get())
	res = p.set_sqr1(fr)

w = Tk()
Label(text=_('Use Slider to change SQR1 from 1 to 100 Hz')).pack(side=TOP)
Sqr1 = Scale(w,command = set_freq, orient=HORIZONTAL, length=220, showvalue=True, from_ = 1, to=100, resolution=.1)
Sqr1.pack(side=TOP)
Button(text=_('QUIT'), command=sys.exit).pack(side=TOP)
#eyeplot.pop_image('pics/driven-pend.png', _('Driven Pendulum'))
w.title(_('EYES Junior: Driven Pendulum'))
w.mainloop()




from Tkinter import *
import expeyes.eyesj
p=expeyes.eyesj.open()

import gettext
gettext.bindtextdomain("expeyes")
gettext.textdomain('expeyes')
_ = gettext.gettext

def set_freq(w):
	state = int(Sqr1.get())
	fr = float(Sqr1.get())
	res = p.set_sqr1(fr)

w = Tk()
Label(text=_('Use Slider to change SQR1 from 1 to 50 Hz')).pack(side=TOP)
Sqr1 = Scale(w,command = set_freq, orient=HORIZONTAL, length=220, showvalue=True, from_ = 1, to=50, resolution=.1)
Sqr1.pack()
w.title(_('EYES Junior: Set SQR1'))
w.mainloop()




'''
expEYES program
Developed as a part of GSoC Project 
this program allows user to use TWO motion sensors (srf-05 modules) and plot graphs in real-time.
'''
import gettext
gettext.bindtextdomain("expeyes")
gettext.textdomain('expeyes')
_ = gettext.gettext

from Tkinter import *
import time, sys, math
if sys.version_info.major==3:
from tkinter import *
else:
from Tkinter import *
sys.path=[".."] + sys.path
import expeyes.eyesj as eyes
import expeyes.eyeplot as eyeplot
import expeyes.eyemath as eyemath



WIDTH = 600 # width of drawing canvas
HEIGHT = 400 # height
vs = 0.034000
class Pend:
tv = [ [], [], [] ] # Lists for Readings
TIMER = 10	# Time interval between reads
MINY = 0	
MAXY = 80
running = False
MAXTIME = 10
def xmgrace(self):
if self.running == True:
return
p.grace([self.tv])
def start(self):
self.running = True
self.index = 0
self.tv = [ [], [], [] ]
try:
self.MAXTIME = int(DURATION.get())
self.MAXY = int(MAXDIST.get())
g.setWorld(0, self.MINY, self.MAXTIME, self.MAXY,_('Time'),_('cm'))
Dur.config(state=DISABLED)
self.msg(_('Starting the Measurements'))
root.after(self.TIMER, self.update)
except:
self.msg(_('Failed to Start'))
def stop(self):
self.running = False
Dur.config(state=NORMAL)
self.msg(_('User Stopped the measurements'))
def update(self):
if self.running == False:
return
tt = p.srfechotime(9,0)
dist = (tt-400) *vs/2
tt2 = p.srfechotime(8,3)
dist2 = (tt2-400) *vs/2
if len(self.tv[0]) == 0:
self.start_time = time.time()
elapsed = 0
else:
elapsed = time.time() - self.start_time
self.tv[0].append(elapsed)
self.tv[1].append(dist)
self.tv[2].append(dist2)
if len(self.tv[0]) >= 2:
g.delete_lines()
g.line(self.tv[0], self.tv[1])
g.line(self.tv[0], self.tv[2],1)
if elapsed > self.MAXTIME:
self.running = False
Dur.config(state=NORMAL)
self.msg(_('Completed the Measurements'))
return
root.after(self.TIMER, self.update)
def save(self):
try:
fn = filename.get()
except:
fn = 'motion2.dat'
p.save([self.tv],fn)
self.msg(_('Data saved to %s')%fn)
def clear(self):
if self.running == True:
return
self.tv = [ [], [] ]
g.delete_lines()
self.msg(_('Cleared Data and Trace'))
def msg(self,s, col = 'blue'):
msgwin.config(text=s, fg=col)
p = eyes.open()
p.disable_actions()
root = Tk()
Canvas(root, width = WIDTH, height = 5).pack(side=TOP) # Some space at the top
g = eyeplot.graph(root, width=WIDTH, height=HEIGHT, bip=False) # make plot objects using draw.disp
pen = Pend()
cf = Frame(root, width = WIDTH, height = 10)
cf.pack(side=TOP, fill = BOTH, expand = 1)
b3 = Label(cf, text = _('Digitize for'))
b3.pack(side = LEFT, anchor = SW)
DURATION = StringVar()
Dur =Entry(cf, width=5, bg = 'white', textvariable = DURATION)
DURATION.set('10')
Dur.pack(side = LEFT, anchor = SW)
b3 = Label(cf, text = _('Seconds.'))
b3.pack(side = LEFT, anchor = SW)
b3 = Label(cf, text = _('Max Dist='))
b3.pack(side = LEFT, anchor = SW)
MAXDIST = StringVar()
Dis =Entry(cf, width=5, bg = 'white', textvariable = MAXDIST)
MAXDIST.set('60')
Dis.pack(side = LEFT, anchor = SW)
b3 = Label(cf, text = _('cm'))
b3.pack(side = LEFT, anchor = SW)
cf = Frame(root, width = WIDTH, height = 10)
cf.pack(side=TOP, fill = BOTH, expand = 1)
b1 = Button(cf, text = _('START'), command = pen.start)
b1.pack(side = LEFT, anchor = N)
b1 = Button(cf, text = _('STOP'), command = pen.stop)
b1.pack(side = LEFT, anchor = N)
b4 = Button(cf, text = _('CLEAR'), command = pen.clear)
b4.pack(side = LEFT, anchor = N)
b1 = Button(cf, text = _('Xmgrace'), command = pen.xmgrace)
b1.pack(side = LEFT, anchor = N)
b3 = Button(cf, text = _('SAVE to'), command = pen.save)
b3.pack(side = LEFT, anchor = N)
filename = StringVar()
e1 =Entry(cf, width=15, bg = 'white', textvariable = filename)
filename.set('motion2.dat')
e1.pack(side = LEFT)
b5 = Button(cf, text = _('QUIT'), command = sys.exit)
b5.pack(side = RIGHT, anchor = N)
mf = Frame(root, width = WIDTH, height = 10)
mf.pack(side=TOP)
msgwin = Label(mf,text=_('Message'), fg = 'blue')
msgwin.pack(side=LEFT, anchor = S, fill=BOTH, expand=1)
#eyeplot.pop_image('pics/imagename.png', _('write the name of expt setup image'))
root.title(_('TWO Motion sensors'))
root.mainloop()



from __future__ import print_function
import expeyes.eyesj,time

p=expeyes.eyesj.open()

p.set_state(10,1)
x=0
while 1:
	#n = input('Enter number(0 to 255) :')
	p.irsend1(x)
	time.sleep(.5)
	print (x)
	x+=1



'''
Plotting libray, using Qt5 for expEYES
Author  : Georges Khaznadar <georgesk@debian.org>
Based on Ajith Kumar's work
License : GNU GPL version 3
'''

from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *

AXWIDTH = 40   # width of the axis display canvas
AYWIDTH = 50   # width of the axis display canvas
NUMDIV  = 5
NGRID1  = 10
NGRID2  = 10
BGCOL     = Qt.white
PLOTBGCOL = Qt.white
LINEWIDTH = 2 
LINECOL   = [Qt.black, Qt.red, Qt.blue, Qt.green, Qt.cyan, Qt.magenta,
             Qt.yellow, Qt.darkYellow,Qt.gray, Qt.darkGray]
LABELCOL  = Qt.blue
TEXTCOL   = Qt.black

GRIDCOL   = Qt.gray
NGRID1    = 10
NGRID2    = 5

class plotWidget(QGraphicsView):
    """
    a widget to plot measurements coming from Expeyes box.
    it features x and y axis, and can manage scaling commands
    """
    border = 2
    pad = 0
    bordcol = Qt.gray     # Border color
    gridcol = Qt.gray     # Grid color
    bgcolor = '#dbdbdb'  # background color for all 
    plotbg  = 'ivory'    # Plot window background color
    textPen = QPen(Qt.blue)
    labelPen = QPen(LABELCOL)
    dotGridPen = QPen(gridcol)
    solidGridPen = QPen(gridcol)
    traces = []
    gridLines=[]
    xaxis = []
    yaxis = []
    legendtext = []
    scaletext = []
    markerval = []
    markertext = None

    def __init__(self, parent=None, width=400., height=300.,color = 'white', labels = True, bip=True, xAxisWidget=None, yAxisWidget=None):
        """
        constructor.
        @param parent a widget (default=None)
        @param width width of the graph (default=400.0)
        @param height height of the graph (default=300.0)
        @param color background color
        @param labels True to display labels (default=True)
        @param bip True if the O V is in the middle (default=True)
        @param xAxisWidget a widget to draw abscissa labels and ticks
        @param yAxisWidget a widget to draw ordinate labels and ticks        
        """
        QGraphicsView.__init__(self, parent)
        self.parent = parent
        self.labels = labels
        self.SCX = width 
        self.SCY = height
        self.plotbg = color
        self.bipolar = bip
        self.scene = QGraphicsScene(self)
        self.scene.setSceneRect(QRectF(0, 0, self.SCX, self.SCY))
        self.setScene(self.scene)
        self.labelFont=QFont("Courier New")
        self.labelFont.setPixelSize(11)
        self.dotGridPen.setStyle(Qt.DotLine)
        self.solidGridPen.setStyle(Qt.SolidLine)
        #TODO: pack self into the parent

        self.setWorld(0 , 0, self.SCX, self.SCY)
        self.grid()

    def setWorld(self, x1, y1, x2, y2, xUnit='ms', yUnit='V'):
        '''
        Calculates the scale factors for world to screen
        coordinate transformation.
        @param x1 lowest value of x to plot
        @param y1 lowest value of y to plot
        @param x2 highest value of x to plot
        @param y2 highest value of y to plot
        @param xUnit the unit to use for abscissa
        @param yUnit the unit to use for ordinate
        '''
        self.xmin = float(x1)
        self.ymin = float(y1)
        self.xmax = float(x2)
        self.ymax = float(y2)
        self.xscale = (self.xmax - self.xmin) / (self.SCX)
        self.yscale = (self.ymax - self.ymin) / (self.SCY)   
        #self.mark_labels(xUnit, yUnit)
        if self.labels == True:
            return
        for txt in self.scaletext:
            self.scene.removeItem(txt)
        self.scaletext = []
        s = '%3.2f %s/div'%( (self.xmax-self.xmin)/NGRID1, xUnit)
        t =  self.scene.addSimpleText(s)
        t.setPos(QPointF(2, self.SCY*9/20))
        t.setPen(self.labelPen)
        self.scaletext.append(t)
        s = '%3.2f %s/div'%( (self.ymax-self.ymin)/NGRID1, yUnit)
        t =  self.scene.addSimpleText(s)
        t.setPos(QPointF(self.SCX/2,10))
        t.setPen(self.labelPen)
        self.scaletext.append(t)

    def mark_labels(self, xUnit, yUnit):
        '''
        Draws the X and Y axis divisions and labels. Only used internally.
        @param xUnit the unit to use for xaxis
        @param yUnit the unit to use for yaxis
        '''
        if self.labels == False:
            return

        self.xaxis = []
        self.yaxis = []
        pos=QPoint(self.SCX/2, self.SCY-AXWIDTH+15)
        self.xaxis.append((self.textPen, pos, self.labelFont, xUnit, "right", "bottom" ))
        dx = float(self.SCX)/NUMDIV
        for x in range(0,NUMDIV+1):
            a = x *(self.xmax - self.xmin)/NUMDIV + self.xmin
            s = '%4.1f'%(a)
            adjust = 0
            if x == 0: adjust = 6
            if x == NUMDIV: adjust = -10
            pos=QPoint(x*dx+adjust, self.SCY-AXWIDTH)
            self.xaxis.append((self.textPen, pos, self.labelFont, s, "center", "bottom"))
        pos=QPoint(2,(self.SCY-AXWIDTH)/2)
        self.yaxis.append((self.textPen, pos, self.labelFont, yUnit, "right", "top"))
        dy = float(self.SCY)/NUMDIV
        for y in range(0,NUMDIV+1):
            a = y*(self.ymax - self.ymin)/5    # + self.ymin
            if self.ymax > 99:
                s = '%4.0f'%(self.ymax-a)
            else:
                s = '%4.1f'%(self.ymax-a)
            if y == 0:
                shift=5
            elif y == NUMDIV:
                shift = -6
            else:
                shift = 0
            pos=QPoint(AYWIDTH, int(y*dy)+shift)
            self.yaxis.append((self.textPen, pos, self.labelFont, s, "center", "center"))

    def polyline(self, x, y, pen=0, replaces=None):
        """
        draws a polyline in the grid with the coordinates of the world
        @param x a vector of abscissas
        @param y a vector of ordinates
        @param pen a QPen, or an index to colors in LINECOL
        @param replaces the polyline to replace if any
        @return a reference to the tuple (pen, polygon) which was issued
        """
        if replaces:
            self.traces.delete(replaces)
        ip = self.w2s(x,y)
        pol=QPolygonF(ip)
        if not isinstance(pen,QPen):
            pen=QPen(LINECOL[pen%len(LINECOL)])
        self.traces.append((pen,pol))
        self.update()

    def delete_lines(self):
        """
        removes all the traces
        """
        self.traces=[]
        return

    def paintEvent(self, event=None):
        """
        redefinition of the SLOT which deals with paint events (raised
        by repaint or update)
        @param event the paint event
        """
        QGraphicsView.paintEvent(self, event)
        painter=QPainter(self.viewport())
        painter.save()
        for pen, pos, font, text, xlayout, ylayout in self.xaxis + self.yaxis:
            painter.setPen(pen)
            painter.setFont(font)
            fm=painter.fontMetrics()
            w=fm.width(text)
            h=fm.height()
            x=pos.x()
            y=pos.y()
            if xlayout == "left":
                x-=w
            elif xlayout == "center":
                x-=w/2
            if ylayout == "bottom":
                y+=h
            elif ylayout == "center":
                y+=h/2
            pos=QPoint(x,y)
            painter.drawText(pos, text)
        for pen, pol in self.gridLines + self.traces:
            painter.setPen(pen)
            painter.drawPolyline(pol)
        painter.restore()

    def grid(self):
        dx = (self.xmax - self.xmin) / NGRID1
        dy = (self.ymax - self.ymin) / NGRID1
        x = self.xmin + dx
        #print(self.ymin)
        if self.bipolar == True:
            ip = self.w2s((self.xmax/2,self.xmax/2),(self.ymin,self.ymax))
            self.gridLines.append((self.solidGridPen,QPolygonF(ip)))
            ip = self.w2s((self.xmin,self.xmax),(self.ymax/2,self.ymax/2))  
            self.gridLines.append((self.solidGridPen,QPolygonF(ip)))

        
        while x < self.xmax:
            ip = self.w2s((x,x),(self.ymin,self.ymax))  
            self.gridLines.append((self.dotGridPen,QPolygonF(ip)))
            x = x +dx
        y = self.ymin + dy
        while y < self.ymax:
            ip = self.w2s( (self.xmin,self.xmax), (y,y) )
            self.gridLines.append((self.dotGridPen,QPolygonF(ip)))
            y = y +dy

    def w2s(self, x,y):
        """
        World to Screen xy conversion before plotting anything
        @param x a vector of abscissas
        @param y a vector of ordinates (same length)
        @return a list of QPoints converted to screen coordinates
        """
        ip = []
        for i in range(len(x)):
            ix = self.border + int( (x[i] - self.xmin) / self.xscale)
            iy = self.border + int( (y[i] - self.ymin) / self.yscale)
            iy = self.SCY - iy
            ip.append(QPointF(ix,iy))
        return ip

                          

if __name__ == '__main__':
    import numpy as np
    import sys
    
    app = QApplication(sys.argv)
    view = plotWidget(labels=False)
    view.show()
    nbPoint=100
    x=np.fromiter((400.0*x/nbPoint for x in range(nbPoint+1)), dtype=np.float)
    def myfunc(a):
        return 100*np.sin(a/20)+150
    vecfunc=np.vectorize(myfunc)
    y=vecfunc(x)
    view.polyline(x,y)
    sys.exit(app.exec_())



'''
expEYES Junior calibration program
Author  : Ajith Kumar B.P, bpajith@gmail.com
License : GNU GPL version 3
'''
from __future__ import print_function

import gettext, sys, os, os.path, time 
if sys.version_info.major==3:
        from tkinter import *
else:
        from Tkinter import *

sys.path=[".."] + sys.path

import expeyes.eyesj as eyes
import expeyes.eyeplot as eyeplot
import expeyes.eyemath as eyemath

gettext.bindtextdomain("expeyes")
gettext.textdomain('expeyes')
_ = gettext.gettext

from numpy import mean, zeros

def msg(s, col='blue'):
	msgwin.config(text=s, fg=col)

def save_calib():	# Saves scale factors of A1 & A2 to calibrationFile
	v = p.set_voltage(2.0)
	v1 = p.get_voltage(1)
	if abs(v1-v) > 0.1:
		msg(_('PVS is NOT connected to A1'),'red')
		return
	v2 = p.get_voltage(2)
	if(abs(v2-v) > 0.1):
		msg(_('PVS is NOT connected to A2'), 'red')
		return

	np = 10
	x = zeros(np)				# Calibrate A1 
	y  = zeros(np)
	iv = 50						# DAC binary value
	for k in range(np):
		p.write_dac(iv)
		time.sleep(0.01)
		x[k] = p.read_adc(1)				# binary from A1, after level shifting
		y[k] = p.read_adc(12)*p.refval/4095		# voltage at PVS (connected to A1)
		iv += 350
	xbar = mean(x)							# Calculate m & c for A1  , 12 bit
	ybar = mean(y)
	m1 = sum(y*(x-xbar)) / sum(x*(x-xbar))
	m1 = float(m1)
	c1 = ybar - xbar * m1
	c1 = float(c1)
	# Do some sanity check here
	ucm = 10.0/4095		# Uncalibrated values of m and c
	ucc = -5.0
	dm = ucm * 0.03			# maximum 3% deviation
	dc = 5 * 0.03
	if abs(m1 - ucm) > dm or abs(c1 - ucc) > dc:
		msg(_('Too much error in A1: m = %f  c=%f')%(m1,c1),'red')
		return

	iv = 10						# Calibrate A2
	for k in range(np):
		p.write_dac(iv)
		time.sleep(0.01)
		x[k] = p.read_adc(2)				# binary from A2, after level shifting
		y[k] = p.read_adc(12)*p.refval/4095	# voltage at PVS (connected to A2)			
		iv += 350
	xbar = mean(x)							# Calculate m & c for A1  , 12 bit
	ybar = mean(y)
	m2 = sum(y*(x-xbar)) / sum(x*(x-xbar))
	m2 = float(m2)
	c2 = ybar - xbar * m2
	c2 = float(c2)
	if abs(m2 - ucm) > dm or abs(c2 - ucc) > dc:	# Error check
		msg(_('Too much error in A2: m = %f  c=%f')%(m2,c2),'red')
		return
	print (m1,c1,m2,c2)
	if p.storeCF_a1a2(m1, c1, m2, c2) == None:	# Store to EEPROM
		msg(_('EEPROM write failed. Old Firmware ?'),'red')
		return	
	ss =_('m1 = %f   c1 = %6.3f\nm2 = %f   c2 = %6.3f')%(m1, c1, m2, c2)
	msg(_('A1&A2 Calibration Saved to EEPROM\n')+ss)
	print (ss)

#------------------------------------------------------------------------------
def save_calibref():	# Saves reference voltage
	try:
		v = float(Rref.get())
	except:
		msg(_('Enter the valid voltage'), 'red')
		return
	print (v)
	if (v < 4.950) or (v > 5.050):
		msg(_('Too much error in reference %5.3f voltage')%v, 'red')
		return
	if p.storeCF_ref(v) == None:					# Store to EEPROM
		msg(_('EEPROM write failed. Old Firmware ?'),'red')
		return
	else:
		msg(_('Calibrated Reference. Vref =%5.0f')%v )

#------------------------------------------------------------------------------
def save_calibsen():	# Saves scale factors of A1 & A2 to file 'eyesj.cal'
	v = p.get_voltage(5)
	try:
		r = float(Rsen.get())
		R = r * (5.0 - v) / v
	except:
		msg(_('Enter the R connected to SEN'), 'red')
		return
	print (R)
	if (R < 4950) or (R > 5250):
		msg(_('Wrong Resistor ??. Calculated Rp =%5.1f Ohm')%R, 'red')
		return
	if p.storeCF_sen(R) == None:					# Store to EEPROM
		msg(_('EEPROM write failed. Old Firmware ?'),'red')
		return
	else:
		msg(_('Calibrated SEN. Rp =%5.0f')%R )

#------------------------- Capacitance calibration ------------------------
socket_cap = 0

def measure_socketcap():
	global socket_cap
	sc = p.measure_cap_raw()
	if 10 < sc < 60:
		socket_cap = sc
		msg(_('Empty Socket is %5.1f pF')%socket_cap)
	else:
		msg(_('IN1 not empty'), 'red')

def save_calibcap():
	global socket_cap
	if socket_cap == 0:
		msg(_('Measure Socket capacitance first'), 'red')
		return
	try:
		creal = float(Cin.get())
	except:
		msg(_('Enter the C connected to IN1'), 'red')
		return
	try:
		cm = p.measure_cap_raw() - socket_cap
		error = creal/cm
	except:
		msg(_('Mesuring capacitor failed'),'red')
		return

	print (creal, cm, error, cm*error)
	if error < 0.7 or error > 1.3 or socket_cap > 60:	# Error check
		msg(_('Too much error: Socket C= %f CF=%f')%(socket_cap, error),'red')
		return
	if p.storeCF_cap(socket_cap, error) == None:		# Store to EEPROM
		msg(_('Write to EEPROM failed'),'red')
	else:
		msg(_('Saved: Socket C = %5.1f pF. CF = %5f %%')%(socket_cap, error))

#---------------------------------------------------------------------------------
root = Tk()
Label(root, text = _('Reference Calibration'),font=('Helvetica', 14),\
		fg='blue').pack(side=TOP)
f = Frame(root, relief = SUNKEN)
f.pack(side=TOP)
Label(f, text = _('Enter measured value of Vref')).pack(side=LEFT)
Refval = StringVar()
Rref = Entry(f, width=6, textvariable=Refval)
Rref.pack(side=LEFT)
Refval.set('5.000')
Label(f, text = _('V')).pack(side=LEFT)
Button(root,text = _("and Click here to Set Reference"), \
	command = save_calibref).pack(side = TOP)
separator = Frame(height=2, bd=1, relief=SUNKEN)

separator.pack(fill=X, padx=5, pady=5)
Label(root, text = _('Calibration of Inputs A1 & A2'), fg='blue',\
			font=('Helvetica', 14)).pack(side=TOP)
Label(root, text = _('Connect PVS to both A1 and A2')).pack(side=TOP)
Button(root, text = _("Calibrate A1 & A2"), command = save_calib).pack(side = TOP)
separator = Frame(height=2, bd=1, relief=SUNKEN)
separator.pack(fill=X, padx=5, pady=5)

Label(root, text = _('Calibration of Resistor on SEN'),font=('Helvetica', 14),\
		fg='blue').pack(side=TOP)
f = Frame(root, relief = SUNKEN)
f.pack(side=TOP)
Label(f, text = _('Enter the Resistance connected from SEN to GND=')).pack(side=LEFT)
Rval = StringVar()
Rsen = Entry(f, width=6, textvariable=Rval)
Rsen.pack(side=LEFT)
#Rval.set('4984')
Label(f, text = _('Ohm')).pack(side=LEFT)
Button(root,text = _("and Click here to Calibrate SEN"), \
	command = save_calibsen).pack(side = TOP)
separator = Frame(height=2, bd=1, relief=SUNKEN)
separator.pack(fill=X, padx=5, pady=5)

Label(root, text = _('Calibration of IN1 for Capacitance'),font=('Helvetica', 14),\
		fg='blue').pack(side=TOP)
Button(root,text = _("First, Click Here without Capacitor on IN1"), \
	command = measure_socketcap).pack(side = TOP)
f = Frame(root)
f.pack(side=TOP)
Label(f, text = _('Enter the Capacitance connected to IN1')).pack(side=LEFT)
Cval = StringVar()
Cin = Entry(f, width=6, textvariable = Cval)
Cin.pack(side=LEFT)
#Cval.set('980')
Label(f, text = _('pF')).pack(side=LEFT)
Button(root,text = _("and Click Here to Calibrate IN1"), command = save_calibcap).pack(side = TOP)
separator = Frame(height=2, bd=1, relief=SUNKEN)
separator.pack(fill=X, padx=5, pady=5)

msgwin = Label(root, text = '')
msgwin.pack(side=TOP)
separator = Frame(height=2, bd=1, relief=SUNKEN)
separator.pack(fill=X, padx=5, pady=5)

Button(text = _("Exit"), command = sys.exit).pack(side = TOP)

p = expeyes.eyesj.open()
if p == None:
	root.title(_('EYES Junior Hardware not found'))
eyeplot.pop_image('pics/calibrate.png', _('Calibrate A1 & A2'))

cfvals = _('Existing calibration Data\n')+\
_('Ref=%5.3fV\nm1=%6.3f c1=%6.3f\nm2=%6.3f c2=%6.3f\nIN1: CF=%6.3f Socket=%6.3fpF\n SEN pullup=%6.1fOhm')\
		%(p.refval,p.m12[1], p.c[1],p.m12[2], p.c[2], p.cap_calib, p.socket_cap,p.sen_pullup)
msg(cfvals)
root.title(_('EYES Junior Calibration'))
root.mainloop()




import sys

if sys.version_info.major==3:
        from tkinter import *
else:
        from Tkinter import *

sys.path=[".."] + sys.path
import expeyes.eyesj, expeyes.eyeplot as eyeplot

p=expeyes.eyesj.open()

import gettext
gettext.bindtextdomain("expeyes")
gettext.textdomain('expeyes')
_ = gettext.gettext

def set_pvs(w):
	state = int(Pvs.get())
	iv = int(Pvs.get())
	p.write_dac(iv)
	v = p.get_voltage(12)
	Res.config(text=_('PVS = %5.3f volts')%v)
w = Tk()
Label(text=_('To change PVS drag the slider. For fine adjustment, click on its left or right')).pack(side=TOP)
Pvs = Scale(w,command = set_pvs, orient=HORIZONTAL, length=500, showvalue=False, from_ = 0, to=4095, resolution=1)
Pvs.pack(side=TOP)
Res = Label(text = '', fg = 'blue')
Res.pack(side=TOP)
Button(text=_('QUIT'), command=sys.exit).pack(side=TOP)
w.title(_('EYES Junior: Adjust PVS'))
w.mainloop()




#!/usr/bin/python

import re,os, datetime, shelve

d=shelve.open(os.environ['EXPEYES_SHELVE'],"r")
data=d["data"]
title=d["title"]
xlabel=d["xlabel"]
ylabel=d["ylabel"]
d.close()
os.remove(os.environ['EXPEYES_SHELVE'])

ncol=len(data)
t = newTable("CRO", len(data[0]), ncol)

i=0
for col in data:
    t.setColData((1+i), tuple(col))
    i+=1


g = newGraph("CRO", 1)
l = g.activeLayer()

l.setTitle("%s (%s)" % (title, datetime.datetime.now()))
l.setTitleFont(QtGui.QFont("Arial", 12))
l.setAxisTitle(Layer.Bottom, xlabel)
l.setAxisTitle(Layer.Left, ylabel)

cn=t.colNames()
l.insertCurve(t, cn[0], cn[1], Layer.Line)# returns a reference to the inserted curve
for cname in cn[2:]:
    l.addCurve(t, cname, Layer.Line)



'''
expEYES Junior CRO+ program
Author  : Ajith Kumar B.P, bpajith@gmail.com
License : GNU GPL version 3
Date : Apr-2012
'''
from __future__ import print_function

import gettext
gettext.bindtextdomain("expeyes")
gettext.textdomain('expeyes')
_ = gettext.gettext

import sys
sys.path=[".."] + sys.path
if sys.version_info.major==3:
    from tkinter import *
else:
    from Tkinter import *

import expeyes.eyesj as eyes
import expeyes.eyeplot as eyeplot
import expeyes.eyemath as eyemath
import time, os

bgcol = 'ivory'

BUFSIZE = 1800		# uC buffer size in bytes
TIMER = 100
WIDTH  = 500   		# width of drawing canvas  (make 550)
HEIGHT = 400   		# height 
VPERDIV = 1.0		# Volts per division, vertical scale
NP = 400			# Number of samples
NC = 1				# Number of channels
MINDEL = 4		
delay = MINDEL		# Time interval between samples
CMERR = False

MAXCHAN = 4
chan4 = [ [1, [], [],0,[],0,0,0,None,None,0.0 ],\
          [0, [], [],0,[],0,0,0,None,None,0.0 ],\
          [0, [], [],0,[],0,0,0,None,None,0.0 ],\
          [0, [], [],0,[],0,0,0,None,None,0.0 ]\
]
# Source, t, v, fitflag, vfit, amp, freq, phase, widget1, widget2, display offset in volts
CHSRC   = 0		# index of each item in the list above.
TDATA   = 1
VDATA   = 2
FITFLAG = 3
VFDAT   = 4
AMP     = 5
FREQ    = 6
PHASE   = 7
WINFO   = 8
WFIT    = 9
DOFFSET = 10

# Data Sources and their names
sources = ['A1','A2','IN1', 'IN2', 'SEN', 'SQ1', 'SQ2', 'OD1', 'CCS']
channels = ['CH1', 'CH2', 'CH3', 'CH4']
chancols = ['black', 'red', 'blue','magenta']
# Actions before capturing waveforms
actions = ['ATR', 'WHI', 'WLO', 'WRE', 'WFE','SHI', 'SLO', 'HTP', 'LTP']
acthelp = [_('Analog Trigger'),_('Wait for HIGH'), _('Wait for LOW'),
	   _('Wait for Rising Edge'), _('Wait for Falling Edge'),
	   _('Set HIGH'), _('Set LOW'), _('High True Pulse'),
	   _('Low True Pulse')
]

srchelp = [
  _('Analog Input -5 to +5 volts. Drag this to CH1 .. CH4 to Display it'),
  _('Analog Input -5 to +5 volts. Drag this to CH1 .. CH4 to Display it'),
  _('Analog Input  0 to +5 volts. Drag this to CH1 .. CH4 to Display it'),
  _('Analog Input  0 to +5 volts. Drag this to CH1 .. CH4 to Display it'),
  _('Analog Input  0 to +5 volts. Drag this to CH1 .. CH4 to Display it'),
  _('Analog Input  0 to +5 volts. Drag this to CH1 .. CH4 to Display it'),
  _('Analog Input  0 to +5 volts. Drag this to CH1 .. CH4 to Display it'),
  _('Digital Output 0 to +5 volts. SHI, SLO,HTP or LTP can be assigned to this'),
  _('Constant Current Source Output. SHI, SLO,HTP or LTP can be assigned to this')
]

# Geometry of the left panel, selection of triggers  & channels
LPWIDTH  = 80
LPHEIGHT = 320
VSTEP    = 25
VBORD    = 10
OFFSET   = VSTEP * len(sources)
SELSRC   = 1
SETACT   = 2
WAITACT  = 3
SELCHAN  = 4
NORMAL   = 100	      # Status of Display channel
FIT      = 101	      # Fit to Sinusoid
DEL      = 102	      # Remove entry
FTR      = 103	      # Fourier transform
selection = 0
seltag    = ''	      # selected tag

def set_ch1_offset(val):
    chan4[0][DOFFSET] = int(val) * VPERDIV

def set_ch2_offset(val):
    chan4[1][DOFFSET] = int(val) * VPERDIV

def set_ch3_offset(val):
    chan4[2][DOFFSET] = int(val) * VPERDIV

def set_ch4_offset(val):
    chan4[3][DOFFSET] = int(val) * VPERDIV

def show_ftr(ch):
    fa = eyemath.fit_sine(chan4[ch][TDATA],chan4[ch][VDATA])	# get frequency to decide suitable 'dt'
    if fa != None:
        fr = fa[1][1]*1000        # frequency in Hz
        dt = int(1.e6/ (20 * fr)) # dt in usecs, 20 samples per cycle
        t,v = p.capture(chan4[ch][CHSRC], 1800, dt)
        xa,ya = eyemath.fft(v,dt)
        eyeplot.plot(xa*1000,ya,
                     title = _('Fourier Transform,power spectrum'),
                     xl = _('Freq'), yl = _('Amp'))
        eyeplot.plot(t,v,
                     title = _('Original signal resampled'),
                     xl = _('ms'), yl = _('V'))
        msg(_('%s Fourier transform done, Data saved to "cro.dat" and "fft.dat"')
            % (channels[seltag]))
        p.save([[t,v]], 'cro.dat')
        p.save([[xa,ya]],'fft.dat')

def msg(s, col='blue'):
    msgwin.config(text=s, fg=col)

def set_timebase(w):
    global delay, NP, NC, VPERDIV, chan4
    divs = [0.100, 0.200, 0.500, 1.0, 2.0, 5.0, 10.0, 20.0, 50.0,100.0]
    msperdiv = divs[int(timebase.get())]
    totalusec = int(msperdiv * 1000 * 10)
    chans = []             # Update channel & color information
    for m in range(len(chan4)):
        if chan4[m][0] > 0:
            chans.append(chan4[m][0])  # channel number
    NC = len(chans)
    if NC < 1:
        return
    if totalusec == 1000:
        NP = 250
        delay = 4*NC
    else:
        NP = 400
        delay = (totalusec/NP)*NC

    if delay < MINDEL*NC:
        delay = MINDEL*NC
    elif delay > 1000:
        delay = 1000

    totalmsec = round(0.001 * NP * NC *delay)
    tms = int(totalmsec)
    NP = tms * 1000/NC/delay
    if NP%2 == 1 : NP += 1  # Must be an even number, for fitting
    if NP > 450: NP = 450
    g.setWorld(0,-5*VPERDIV, NP * delay * 0.001, 5*VPERDIV,_('mS'),'V')
    #print (_('NP delay = '),NP, delay, 0.0001 * NP*delay, msperdiv)

def measure_freq(e):
    w = e.widget
    tag =w.find_closest(e.x, e.y)
    item = w.itemcget(tag,'tag')
    target = int(item.split()[0])
    if e.x < LPWIDTH/2 and e.y < OFFSET and 2 < target < 8:
        # Selected IN1, IN2, SEN, SQR1 or SQR2
        freq = p.get_frequency(target)
        if freq > 0.5:
            r2f = p.r2ftime(target, target)*1.0e-6
            msg(_('%4s : Freq = %5.3f. Duty Cycle = %5.1f %%')
                % (sources[target-1], freq, r2f*freq*100))
            s=_('%4s\n%5.3f Hz\n%5.1f %%')%(sources[target-1], freq, r2f*freq*100)
            g.disp(s)
        else:
            msg(_('No squarewave detected on %4s')
                % (sources[target-1]))
    elif e.x > LPWIDTH/2 and e.y > OFFSET and target < 3:
        # Selected CH1, CH2 or CH3
        if chan4[target][CHSRC] != 0 and chan4[target+1][CHSRC] != 0:  # both channels active
            fa = eyemath.fit_sine(chan4[target][TDATA],
                                  chan4[target][VDATA])
            fb = eyemath.fit_sine(chan4[target+1][TDATA],
                                  chan4[target+1][VDATA])
            if fa != None and fb != None:
                v1 = fa[1][0]
                v2 = fb[1][0]
                f1 = fa[1][1]*1000
                # millisecond x-axis gives frequency in kHz, convert it
                f2 = fb[1][1]*1000
                p1 = fa[1][2]
                p2 = fb[1][2]
                s = _('%s: %5.3f V, %5.2f Hz | %s: %5.2f V, %5.3f Hz | Phase difference = %5.1f degree') \
                    % (channels[target], v1, f1, channels[target+1],v2, f2, (p2-p1)*180/3.1416)
                msg(s)
                s=_('%4s\n%5.3f Hz\n%5.1f %%') \
                   % (sources[target-1], freq, r2f*freq*100)
                g.disp(s)
            else:
                msg(_('Fitting of data failed. Try with Xmgrace'))
        else:
            msg(_('Selected channel and the next one should have data'), 'red')

def release(e):
    global selection, seltag, chan4, NC
    w = e.widget
    w.configure(cursor = 'arrow')
    if selection == 0:
        msg(_('Invalid Action'), 'red')
        return
    tag =w.find_closest(e.x, e.y)
    item = w.itemcget(tag,'tag')
    target = int(item.split()[0])
    if e.x > LPWIDTH/2 and e.y > OFFSET and selection == SELSRC:
        # Assign source to channel
        msg(_('Assigned Data Input %4s to Channel %s') %(sources[seltag-1],channels[target]))
        s = '%s:' %(sources[seltag-1])
        chan4[target][WINFO].config(text = s, fg=chancols[target])
        chan4[target][0] = seltag
        set_timebase(0)	# Adding  a channel require recalculation
    elif e.x < LPWIDTH/2 and e.y < OFFSET and selection == SETACT and target > 6:   
        msg(_('%4s effective on Output %s') 
            %(acthelp[seltag], sources[target-1]))
        p.enable_action(seltag, target+2) # There is an offset of 2 for OD1 & CCS
        #print ('SET ', seltag,target+2)
    elif e.x < LPWIDTH/2 and e.y < OFFSET and selection == WAITACT and target <= 7:   
        msg(_('%4s effective on Input %s') 
            %(acthelp[seltag], sources[target-1]))
        p.enable_action(seltag, target)
    elif e.x < LPWIDTH/2 and e.y > OFFSET and selection == SELCHAN:
        # Selected channel    
        if target == DEL:
            chan4[seltag][CHSRC] = 0 
            chan4[seltag][FITFLAG] = 0 
            chan4[seltag][WINFO].config(text = '')
            chan4[seltag][WFIT].config(text = '')
            msg('Disabled Display channel %s'%(channels[seltag]))
            set_timebase(0)				# Deleting a channel require recalculation
        elif target == FIT:
            if chan4[seltag][CHSRC] != 0:
                chan4[seltag][FITFLAG] = True
                msg('Selected %s for fitting'%(channels[seltag]))
            else:
                msg(_('Channel %s is Empty')
                    %(channels[seltag]), 'red')
        elif target == FTR:
            if chan4[seltag][CHSRC] != 0:
                show_ftr(seltag)	# Channel for FT
            else:
                msg(_('Channel %s is Empty') %(channels[seltag]), 'red')
        elif target == NORMAL:
            if chan4[seltag][CHSRC] != 0:
                chan4[seltag][FITFLAG] = False
                chan4[seltag][WFIT].config(text = '')
                msg(_('Disabled fitting %s') %(channels[seltag]))
            else:
                msg(_('Channel %s is Empty')
                    %(channels[seltag]), 'red')
    elif e.x < LPWIDTH/2 and e.y < OFFSET and selection == SELSRC:
        # Selected Source
        src = sources[target-1]
        val = p.get_voltage(target)
        ss = _('Voltage at %s = %5.3f V') %(src,val)
        sm = _('%3s: %5.3f V')%(src, val)
        if 2 < target < 8:
            level = p.get_state(target)
            ss += _(' (Logic Level = %d)') %level
            msg(ss)
            g.disp(sm)
        else:
            msg(_('Invalid selection'), 'red')		

def press(e):
    global selection, seltag
    selection = 0
    w = e.widget
    tag =w.find_closest(e.x, e.y)
    item = w.itemcget(tag,'tag')
    if item == '' or item[0] == 'c': return		# clicked on borders
    seltag = int(item.split()[0])
    if e.x < LPWIDTH/2 and e.y < OFFSET:		# Source selection
        if seltag > 7: 
            msg(_('%4s is an Output') %sources[seltag-1], 'red')
            return
        selection = SELSRC
        msg(_('Selected Data Input %4s. For Trace, Drag this to CH1 .. CH4. To print value release the button.')\
            %sources[seltag-1])
        w.configure(cursor = 'pencil')
    elif e.x > LPWIDTH/2 and e.y < OFFSET:		# Trigger selection
        if seltag >= 5:
            selection = SETACT
            msg(_('Selected %4s. Drag cursor to the OD1 or CSS Output') %acthelp[seltag])
            w.configure(cursor = 'hand1')
        else:
            selection = WAITACT
            msg(_('Selected %4s. Drag cursor to desired Data Input') %acthelp[seltag],)
            w.configure(cursor = 'hand2')
    elif e.x > LPWIDTH/2 and e.y > OFFSET:		# Channel selection
        selection = SELCHAN
        msg(_('Selected %4s. Drag cursor to NML FIT or DEL') %channels[seltag],)
        w.configure(cursor = 'pencil')

def update():
    global delay, NP, NC, VPERDIV, chan4, CMERR
    global NP, NC, delay,chan4
    chans = []                    # Update channel & color information
    index = []
    for m in range(len(chan4)):
        if chan4[m][0] > 0:
            chans.append(chan4[m][0])  # channel number
            index.append(m) # Store the used indices, for storing & fitting
    NC = len( chans)
    
    # force int types as NP and delay can come from float divisions
    # with Python3
    NP=int(NP)
    delay=int(delay)
    
    try:
        if NC == 1:
            chan4[index[0]][TDATA], \
                chan4[index[0]][VDATA] \
                =  p.capture_hr(chans[0], NP, delay)
            v1 = []
            for k in range(NP): v1.append(
                    chan4[index[0]][VDATA][k] +
                    chan4[index[0]][DOFFSET])
            g.delete_lines()
            g.line(chan4[index[0]][TDATA],v1, index[0])
        elif NC == 2:
            chan4[index[0]][TDATA],chan4[index[0]][VDATA], \
                chan4[index[1]][TDATA],chan4[index[1]][VDATA] = p.capture2_hr( chans[0],  chans[1], NP, delay)
            v1 = []
            v2 = []
            for k in range(NP): 
                v1.append(
                    chan4[index[0]][VDATA][k] +
                    chan4[index[0]][DOFFSET])
                v2.append(
                    chan4[index[1]][VDATA][k] +
                    chan4[index[1]][DOFFSET])
            g.delete_lines()
            g.line(chan4[index[0]][TDATA], v1, index[0])
            g.line(chan4[index[1]][TDATA], v2, index[1])
        elif NC == 3:
            chan4[index[0]][TDATA],chan4[index[0]][VDATA], chan4[index[1]][TDATA],chan4[index[1]][VDATA], \
                chan4[index[2]][TDATA],chan4[index[2]][VDATA] = p.capture3( chans[0], chans[1], chans[2], NP, delay)
            v1 = []
            v2 = []
            v3 = []
            for k in range(NP): 
                v1.append(
                    chan4[index[0]][VDATA][k] +
                    chan4[index[0]][DOFFSET])
                v2.append(
                    chan4[index[1]][VDATA][k] +
                    chan4[index[1]][DOFFSET])
                v3.append(
                    chan4[index[2]][VDATA][k] +
                    chan4[index[2]][DOFFSET])
            g.delete_lines()
            g.line(chan4[index[0]][TDATA], v1, index[0])
            g.line(chan4[index[1]][TDATA], v2, index[1])
            g.line(chan4[index[2]][TDATA], v3, index[2])
        elif NC == 4:
            chan4[index[0]][TDATA],chan4[index[0]][VDATA], chan4[index[1]][TDATA],chan4[index[1]][VDATA], \
                chan4[index[2]][TDATA],chan4[index[2]][VDATA], chan4[index[3]][TDATA],chan4[index[3]][VDATA] \
                = p.capture4( chans[0],  chans[1], chans[2], chans[3], NP, delay)
            v1 = []
            v2 = []
            v3 = []
            v4 = []
            for k in range(NP): 
                v1.append(
                    chan4[index[0]][VDATA][k] +
                    chan4[index[0]][DOFFSET])
                v2.append(
                    chan4[index[1]][VDATA][k] +
                    chan4[index[1]][DOFFSET])
                v3.append(
                    chan4[index[2]][VDATA][k] +
                    chan4[index[2]][DOFFSET])
                v4.append(
                    chan4[index[3]][VDATA][k] +
                    chan4[index[3]][DOFFSET])
            g.delete_lines()
            g.line(chan4[index[0]][TDATA], v1, index[0])
            g.line(chan4[index[1]][TDATA], v2, index[1])
            g.line(chan4[index[2]][TDATA], v3, index[2])
            g.line(chan4[index[3]][TDATA], v4, index[3])
        if CMERR == True: 
            CMERR = False
            msg('')
    except:
        msg(_('Communication Error. Check input voltage levels.'),
            'red')
        CMERR = True
	
    for k in range(4):
        if chan4[k][CHSRC] != 0 and chan4[k][FITFLAG] == 1:
            fa = eyemath.fit_sine(chan4[k][TDATA],chan4[k][VDATA])
            if fa != None:
                chan4[k][VFDAT] = fa[0]
                chan4[k][AMP] = abs(fa[1][0])
                chan4[k][FREQ] = fa[1][1]*1000
                chan4[k][PHASE] = fa[1][2] * 180/eyemath.pi
                s = _('%5.2f V, %5.1f Hz')\
                    %(chan4[k][AMP],chan4[k][FREQ])
                chan4[k][WFIT].config(text = s, fg= chancols[k])
    if looping.get() == '0':
        root.after(10,update)

def set_vertical(w):
    global delay, NP, NC, VPERDIV
    divs = [5.0, 1.0, 0.5, 0.2]
    VPERDIV = divs[int(Vpd.get())]
    g.setWorld(0,-5*VPERDIV, NP * delay * 0.001, 5*VPERDIV,_('mS'),'V')

def set_trigger(w):
    tv = Trig.get()
    if p != None: p.set_trigger(tv)

def cro_mode():
    state = looping.get()
    if state == '1':
        Loop.config(text=_('ONE '))
        msg(_('Press SCAN Button to do a single Capture'))
    else:
        Loop.config(text=_('LOOP'))
        root.after(10,update)

def scan():
    if looping.get() == '1':
        update()
        msg(_('Captured %d points in %d usecs') %(NP,NP*delay))
    else:
        msg(_('Use this only in Single scan mode'), 'red')

def set_pvs(e):
    try:
        v = float(Pvs.get())
        res = p.set_voltage(v)
        if res == None:
            msg(_('Enter a value between 0 to +5 volts'),'red')
        else:
            msg(_('PVS set to %5.3f volts') %res)
            g.disp(_('PVS: %5.3f V')%res)
    except:
        msg(_('Enter voltage between -5 and +5 volts'),'red')

def set_sqr1():
    state = int(Sqr1.get())
    if state == 0:
        p.set_sqr1(-1)
        msg(_('SQR1 set to LOW'))
    else:
        try:
            fr = float(Freq.get())
            res = p.set_sqr1(fr)
            if res == None:
                msg(_('Enter a value between .7 to 200000 Hz'))
            else:
                msg(_('SQR1 set to %5.1f Hertz') %res)
        except:
            msg(_('Enter valid frequency, in Hertz'),'red')

def set_sqr2():
    state = int(Sqr2.get())
    if state == 0:
        p.set_sqr2(-1)
        msg(_('SQR2 set to LOW'))
    else:
        try:
            fr = float(Freq.get())
            res = p.set_sqr2(fr)
            if res == None:
                msg(_('Enter a value between .7 to 200000 Hz'))
            else:
                msg(_('SQR2 set to %5.1f Hertz') %res)
        except:
            msg(_('Enter valid frequency, in Hertz'),'red')

def set_sqrs():
    state = int(Both.get())
    if state == 0:
        p.set_sqr1(-1)
        p.set_sqr2(-1)
        msg(_('SQR1 and SQR2 set to LOW'))
    else:
        try:
            fr = float(Freq.get())
            shift = float(Phase.get())
            res = p.set_sqrs(fr,shift)
            if res == None:
                msg(_('Enter a value between .7 to 200000 Hz'))
            else:
                msg(_('SQR1 and SQR2 set to %5.1f Hertz, Shift is %5.2f %% of Period') %(res,shift))
        except:
            msg(_('Enter valid frequency in Hertz and phase shift in percentage'),'red')
            
def sqr1_slider(w):
    if p == None: return
    freq = SQR1slider.get()
    if freq == 0: 
        p.set_sqr1(-1)
        msg(_('SQR1 set to LOW'))
    else:
        fs = p.set_sqr1(freq)
        msg(_('SQR1 set to %5.1f') %fs)

def control_od1():
    state = int(Od1.get())
    p.set_state(10, state)

def control_ccs():
    state = int(Ccs.get())
    p.set_state(11, state)

def measurecap():
    msg(_('Starting Capacitance Measurement..'))
    cap = p.measure_cap()
    if cap == None:
        msg(_('Error: Capacitance too high or short to ground'),'red')
        return
    g.disp(_('IN1: %6.1f pF')%cap)
    if p.socket_cap == 30.0 and p.cap_calib == 1.0:
        msg(_('IN1 Not Calibrated.'))
    else:
        msg(_('IN1: %6.1f pF')%cap)


def measureres():
    res = p.measure_res()
    if res == None:
        msg(p.msg,'red')
        return
    msg(_('Resistance from SEN to GND = %6.0f Ohm')%res)
    g.disp(_('%5.0f Ohm'%res))

def save_data():
    fn = Fname.get()
    dat = []
    for k in range(4):
        if chan4[k][CHSRC] != 0:
            dat.append( [chan4[k][TDATA],chan4[k][VDATA]])
    p.save(dat,fn)
    msg(_('Traces saved to %s') %fn)

def xmgrace():
    dat = []
    for k in range(4):
        if chan4[k][CHSRC] != 0:
            dat.append( [chan4[k][TDATA],chan4[k][VDATA]])
    if p.grace(dat) == False:
        msg(_('Could not find Xmgrace or Pygrace. Install them'),'red')
    else:
        msg(_('Traces send to Xmgrace'))

def process_command(e):
    cp = Result.index(INSERT)
    row = int(cp.split('.')[0])
    ss = Result.get("%d.0"%row, "%d.end"%row)	# User's entry
    cmd = 'p.'+ ss  # command
    p.msg = ''
    try:
        res = eval(cmd)
        if res == None: 
            res = p.msg
    except:
        res = 'Invalid command or argument'
        Result.insert("%d.0"%(row+1), '\n'+str(res)) # Result below

def pop_expt_menu(event):
    poped = True
    menu.post(event.x_root, event.y_root)

def reconnect():
    global p
    import expeyes.eyesj
    p=expeyes.eyesj.open()
    if p == None:
        msg(_('expEYES Junior NOT found. Bad connection or another program using it'),'red')
    else:
        Recon.forget()
        s = _('Four Channel CRO+ found expEYES-Junior on %s') %p.device
        root.title(s)
        msg(s)
        root.after(TIMER,update)

#=================== main program starts here ==================
root = Tk()    
top = Frame(root)
top.pack(side=TOP, anchor =W)
f1 = Frame(top, width = LPWIDTH, height = HEIGHT)
f1.pack(side=LEFT,  fill = BOTH, expand = 1)	# Left side frame

w = Canvas(f1, width=LPWIDTH, height=LPHEIGHT,bg = bgcol)   # Canvas for drag n drop controls
w.pack(side=TOP, anchor = W)
for k in range(len(sources)):
    if k >6: 
        col = 'blue'
    else:
        col = 'black'
    w.create_text (LPWIDTH/2-10, VBORD+k*VSTEP, anchor = E, text = sources[k], tag = k+1, fill=col)
for k in range(len(actions)):
    if  k >= 5:
        col = 'blue'
    else:
        col = 'black'
    w.create_text (LPWIDTH/2+10, VBORD+k*VSTEP, anchor = W, text = actions[k], tag = k, fill=col)
for k in range(4):
    w.create_text (LPWIDTH/2+10, VBORD + OFFSET + k*VSTEP, anchor = W, text = channels[k], tag = k,\
                   fill= chancols[k])
w.create_text (LPWIDTH/2-10, VBORD + OFFSET + 0*VSTEP, anchor = E, text = 'NML', tag = NORMAL)
w.create_text (LPWIDTH/2-10, VBORD + OFFSET + 1*VSTEP, anchor = E, text = 'FTR', tag = FTR)
w.create_text (LPWIDTH/2-10, VBORD + OFFSET + 2*VSTEP, anchor = E, text = 'FIT', tag = FIT)
w.create_text (LPWIDTH/2-10, VBORD + OFFSET + 3*VSTEP, anchor = E, text = 'DEL', tag = DEL)
w.create_line(LPWIDTH/2, 0, LPWIDTH/2, LPHEIGHT)
w.create_line(0, OFFSET, LPWIDTH, OFFSET)
w.bind ("<ButtonPress-1>", press)
w.bind ("<ButtonPress-3>", measure_freq)
w.bind ("<ButtonRelease-1>", release)
offsets = [_('Move UP'), _('CENTER'), _('Move DOWN')]	
offsetmenu = Menu(w, tearoff=0)
for k in range(len(offsets)):
    offsetmenu.add_command(label=offsets[k], background= 'ivory', command = lambda cmd=k :change_offset(cmd))

Label(f1,text = _('mSec/div')).pack(side=TOP, anchor = SW)		# Sliders for Adjusting Axes & Trigger Level
timebase = Scale(f1,command = set_timebase, orient=HORIZONTAL, length=LPWIDTH, showvalue=False,\
                 from_ = 0, to=9, resolution=1)
timebase.pack(side=TOP, anchor = SW)

Label(f1,text = _('Volt/div')).pack(side=TOP, anchor = SW)
Vpd = Scale(f1,command = set_vertical, orient=HORIZONTAL, length=LPWIDTH, showvalue=False,\
            from_ = 0, to=3, resolution=1)
Vpd.pack(side=TOP, anchor = SW)
Vpd.set(1)

Label(f1,text = _('Trig level')).pack(side=TOP, anchor = SW)
Trig = Scale(f1,command = set_trigger, orient=HORIZONTAL, length=LPWIDTH, showvalue=False,\
             from_ = 100, to=4000, resolution=10)
Trig.pack(side=TOP, anchor = SW)
Trig.set(2050)


#----------------------- Middle Frame ------------------------
a = Frame(top, width = LPWIDTH, height = HEIGHT)
a.pack(side=LEFT,  fill = BOTH, expand = 1)
f = Frame(a, width = 75, height = HEIGHT)
f.pack(side=TOP,  fill = BOTH, expand = 1)
g = eyeplot.graph(f, width=WIDTH, height=HEIGHT)	# make plot objects using draw.disp
g.setWorld(0, -5, 20, 5,_('mS'),'V')

#============== Vertical scales for OFFSET adjustment. Lambda not working with Scale callbacks !!! =====
of = Frame(top, width = 1, height = HEIGHT)
of.pack(side=LEFT,  fill = BOTH, expand = 1)

Scale(of, orient=VERTICAL, length=HEIGHT/4, showvalue = False, bg = chancols[0],\
      from_ = 4, to=-4, resolution=1, command = set_ch1_offset).pack(side=TOP)
Scale(of, orient=VERTICAL, length=HEIGHT/4, showvalue = False, bg = chancols[1],\
      from_ = 4, to=-4, resolution=1, command = set_ch2_offset).pack(side=TOP)
Scale(of, orient=VERTICAL, length=HEIGHT/4, showvalue = False, bg = chancols[2],\
      from_ = 4, to=-4, resolution=1, command = set_ch3_offset).pack(side=TOP)
Scale(of, orient=VERTICAL, length=HEIGHT/4, showvalue = False, bg = chancols[3],\
      from_ = 4, to=-4, resolution=1, command = set_ch4_offset).pack(side=TOP)

#================ Right Side panel ====================
rf = Frame(top, width = 75, height = HEIGHT)
rf.pack(side=LEFT,  fill = BOTH, expand = 1)

#---------------------- Extra Features -----------------------------
cf = Frame(rf, border = 1, relief = SUNKEN)
cf.pack(side=TOP,  fill = BOTH, expand = 1)

Label(cf, text = _('Setting Squarewaves'), fg='blue').pack(side=TOP)
f = Frame(cf)
f.pack(side=TOP, anchor = W)
Freq = Entry(f, width = 6)
Freq.pack(side=LEFT, anchor = N)
Freq.insert(0,'1000')
Label(f,text = _('Hz. dphi=')).pack(side=LEFT, anchor = N)
Phase = Entry(f, width=4)
Phase.pack(side=LEFT, anchor=N)
Phase.insert(0,'0')
Label(f,text = '%').pack(side=LEFT, anchor = N)

f = Frame(cf)			# Setting square waves
f.pack(side=TOP)
Sqr1 = IntVar()
Checkbutton(f,text = 'SQR1', command = set_sqr1, variable = Sqr1).pack(side=LEFT, anchor=N)
Sqr2 = IntVar()
Checkbutton(f,text = 'SQR2', command = set_sqr2, variable = Sqr2).pack(side=LEFT, anchor=N)
Both = IntVar()
Checkbutton(f,text = _('BOTH'), command = set_sqrs, variable = Both).pack(side=LEFT, anchor=N)

SQR1slider = Scale(cf,command = sqr1_slider, orient=HORIZONTAL, length=180, showvalue=False, from_ = 0, to=5000, resolution=5)
SQR1slider.pack(side=TOP, anchor=W)
Canvas(cf, height = 5,  width = 100).pack(side=TOP)	# Spacer

f = Frame(cf)			# Setting OD1 and CCS
f.pack(side=TOP)
Label(f, text = _('Set PVS =')).pack(side=LEFT)
Pvs = Entry(f, width = 6)
Pvs.pack(side=LEFT)
Pvs.bind("<Return>",set_pvs)
Pvs.bind("<KP_Enter>",set_pvs)
Pvs.insert(0,'0')
Label(f, text = 'V').pack(side=LEFT)

#Label(cf, text = 'Control OD1 & CCS', fg='blue').pack(side=TOP)
f = Frame(cf)			# Setting OD1 and CCS
f.pack(side=TOP)
Label(f, text = _('Set State')).pack(side=LEFT)
Od1 = IntVar()
Checkbutton(f,text = 'OD1', variable = Od1, command = control_od1).pack(side=LEFT, anchor=N)
Ccs = IntVar()
Checkbutton(f,text = 'CCS', variable = Ccs, command = control_ccs).pack(side=LEFT, anchor=N)
Canvas(cf, height = 5, width= 100).pack(side=TOP)	# Spacer

#ff = Frame(cf)			# Setting OD1 and CCS
#ff.pack(side=TOP)
#Button(ff,text =_('ZeroSet'), command=cap_setzero).pack(side=LEFT, anchor=N)

Button(cf,text =_('Measure C on IN1'), command=measurecap).pack(side=TOP, anchor=N)
Button(cf,text =_('Measure R on SEN'), command=measureres).pack(side=TOP, anchor=N)

Canvas(cf, height = 5, width= 100).pack(side=TOP)	# Spacer

#Label(cf, text = _('Type command<Enter>'), fg='blue').pack(side=TOP)
Result = Text(cf, width = 25, height = 5)
Result.pack(side=TOP)
Result.bind("<Return>", process_command)
Result.bind("<KP_Enter>", process_command)

#-----------------------------------------------------------------
looping = 1				# Make Status display Region
for ch in range(4):
	f = Frame(rf)
	f.pack(side=TOP, anchor = W)
	chan4[ch][8] = Label(f, width=4, text = '', fg=chancols[ch])	# 8 is Label, 9 is value
	chan4[ch][8].pack(side=LEFT, anchor = N)
	chan4[ch][9] = Label(f, width=16, text = '')
	chan4[ch][9].pack(side=LEFT, anchor = N)
chan4[0][WINFO].config(text = 'A1')

#---------------
f = Frame(rf)
f.pack(side=TOP, anchor = W)
Save = Button(f,text=_('Save Traces to'), command = save_data)
Save.pack(side=LEFT, anchor=N)
Fname = Entry(f, width=8)
Fname.pack(side=LEFT)
Fname.insert(0,'cro.txt')

f = Frame(rf)
f.pack(side = TOP, anchor=W)
looping = StringVar()
Loop = Checkbutton(f, text=_('LOOP'), variable = looping, command = cro_mode)
Loop.pack(side=LEFT)
looping.set('0')
Scan = Button(f, text=_('SCAN'), command = scan)
Scan.pack(side=LEFT)
Grace = Button(f, text=_('XMG'), command = xmgrace)
Grace.pack(side=LEFT)

f = Frame(rf)
f.pack(side=TOP, anchor = W)
Expt = Button(f,text = _('EXPERIMENTS'))
Expt.bind("<ButtonRelease-1>", pop_expt_menu)
Expt.pack(side=LEFT)
q = Button(f,text=_('QUIT'), command = sys.exit)
q.pack(side=LEFT, anchor=N)

mf = Frame(root, bg='white')
mf.pack(side=TOP,  fill = BOTH, expand = 1)
msgwin = Label(mf,text = '', justify=CENTER, bg = 'white', fg = 'blue', font=('Helvetica', 12))
msgwin.pack(side=LEFT, anchor = CENTER)
Recon = Button(mf,text = _('Search Hardware'), command =reconnect)

p = eyes.open()
if p == None:
    msg(_('Could not open expEYES Junior. Bad connection or another program using it'),'red')
    Recon.pack(side=LEFT)
else:
	p.disable_actions()
	root.title(_('CRO Plus found expEYES-Junior on %s (firmware: %s)') %(p.device,p.get_version()))
	root.after(TIMER,update)
#------------------------------ experiments menu ------------------------------
expts = [ 
[_('Select Experiment'),''],
[_('Control PVS'),'change-pvs'],
[_('Study of AC Circuits'),'ac-circuit'],
[_('RC Circuit'),'RCcircuit'],
[_('RL Circuit'),'RLcircuit'],
[_('RLC Discharge'),'RLCdischarge'],
[_('EM Induction'),'induction'],
[_('Diode IV'),'diode_iv'],
[_('Transistor CE'),'transistor'],
[_('AM and FM'), 'amfm'],
[_('Frequency Response'),'freq-response'],
[_('Velocity of Sound') , 'velocity-sound'],
[_('Interference of Sound') , 'interference-sound'],
[_('Capture Burst of Sound') , 'sound-burst'],
[_('Driven Pendulum'),'driven-pendulum'],
[_('Rod Pendulum') , 'rodpend'],
[_('Pendulum Wavefrorm'),'pendulum'],
[_('PT100 Sensor'), 'pt100'],
[_('Stroboscope'), 'stroboscope'],
[_('Data Logger'), 'logger'],
[_('HY-SRF05 Echo module'), 'gecho'],
[_('Calibrate'),'calibrate']
 ]

def run_expt(expt):
    global p
    if expt == '': return
    p.fd.close()	# Free the device from this program, the child will open it
    cmd = sys.executable + ' ' + eyeplot.abs_path() + expt+'.py'
    os.system(cmd)
    msg(_('Finished "')+expt+'.py"')
    p = eyes.open()	# Establish hardware communication again, for the parent
    p.disable_actions()

menu = Menu(Expt, tearoff=0)
for k in range(len(expts)):
    text = expts[k][0]
    cmd = expts[k][1]
    #print (text, cmd)
    menu.add_command(label=text, background= 'ivory', command = lambda expt=cmd :run_expt(expt))

root.mainloop()

# Local Variables:
# python-indent: 4
# End:



'''
expEYES Junior CRO+ program
Author  : Ajith Kumar B.P, bpajith@gmail.com
License : GNU GPL version 3
Date : Apr-2012
'''

import gettext, sys
gettext.bindtextdomain("expeyes")
gettext.textdomain('expeyes')
_ = gettext.gettext

if sys.version_info.major==3:
        from tkinter import *
else:
        from Tkinter import *

sys.path=[".."] + sys.path

import expeyes.eyesj as eyes
import expeyes.eyeplot as eyeplot
import expeyes.eyemath as eyemath
import time, os,  math

bgcol = 'ivory'

BUFSIZE = 1800                 # uC buffer size in bytes
TIMER = 100
LPWIDTH = 75
WIDTH  = 600                   # width of drawing canvas
HEIGHT = 400                   # height 
VPERDIV = 1.0                  # Volts per division, vertical scale
NP = 400                       # Number of samples
NC = 1                         # Number of channels
MINDEL = 8                
delay = MINDEL                 # Time interval between samples
CMERR = False
data = [ [[],[]],[[],[]],[[],[]] ]  # 3 [t,v] lists

def msg(s, col='blue'):
    msgwin.config(text=s, fg=col)

def set_timebase(w):
    global delay, NP, NC, VPERDIV, chan4
    divs = [0.100, 0.200, 0.500, 1.0, 2.0, 5.0, 10.0, 20.0, 50.0,100.0]
    msperdiv = divs[int(timebase.get())]
    totalusec = int(msperdiv * 1000 * 10)
    NC = 2
    if totalusec == 1000:
        NP = 250
        delay = 4*NC
    else:
        NP = 400
        delay = (totalusec/NP)*NC

    if delay < MINDEL*NC:
        delay = MINDEL*NC
    elif delay > 1000:
        delay = 1000

    totalmsec = round(0.001 * NP * NC *delay)
    tms = int(totalmsec)
    NP = tms * 1000/NC/delay
    if NP%2 == 1 : NP += 1                # Must be an even number, for fitting
    if NP > 450: NP = 450
    g.setWorld(0,-5*VPERDIV, NP * delay * 0.001, 5*VPERDIV,_('mS'),_('V'))
    msg(_('X-scale changed to %d mS/div.Capturing %d samples with %d usec spacing') %(msperdiv,NP,delay))


def update():
    global delay, NP, delay, VPERDIV,data, CMERR, Freq
    # fix division results when using Python3
    NP=int(NP)
    delay=int(delay)
    if CMERR == True:
        CMERR = False 
        msg('')
    try:
        p.set_trig_source(1)
        t0, v0, t1, v1 = p.capture2_hr(1,2,NP,delay)
        g.delete_lines()
        vp.delete_lines()
        g.line(t0,v0,0)
        g.line(t1,v1,1)
        data[0][0] = t0
        data[0][1] = v0
        data[1][0] = t1
        data[1][1] = v1

        t2 = [0]*NP                # Calculate voltages A1 - A2
        v2 = [0]*NP
        for k in range(NP):
            t2[k] = t1[k]
            v2[k] = v0[k] - v1[k]
        g.line(t2,v2,2)
        data[2][0] = t2
        data[2][1] = v2
        # fitting
        fa = eyemath.fit_sine(t0,v0)
        fb = eyemath.fit_sine(t1,v1)
        fc = eyemath.fit_sine(t2,v2)
        if fa != None and fb != None and fc != None:
            rmsv0 = p.rms(v0)
            dv0 = 100 * abs( (rmsv0-p.rms(fa[0])) /rmsv0 )
            rmsv1 = p.rms(v1)
            dv1 = 100 * abs( (rmsv1-p.rms(fb[0])) /rmsv1 )
            a0 = fa[1][0]
            a1 = fb[1][0]
            a2 = fc[1][0]
            pd01 = math.atan(a2/a1)
            pd01_fit = fb[1][2]-fa[1][2]
            sign = pd01_fit / abs(pd01)
            pherr = abs(pd01) - abs(pd01_fit)
            pherr = abs(pherr/pd01) * 100
            #print 180./math.pi*pd01, 180./math.pi*pd01_fit, pherr
            if dv0 > 2.0 or dv1 > 2.0 or pherr > 100.0:                # Check for error in FIT
                g.line(t0, fa[0], col = 6)
                g.line(t1, fb[0], col = 5)
                msg(_('Error in Fit (A0: Black &Yellow, A1-Red & Green). Try Changing X-scale'))
            # Display even if there is an error in fitted results
            fr = fa[1][1]*1000                
            Fit0.config(text = _('Frequency = %5.1f Hz') %fr)
            Fit1.config(text = _('A1:Total voltage = %5.2f V') %(a0))
            Fit2.config(text = _('A2:Voltage across R = %5.2f V') %(a1))
            Fit3.config(text = _('A1-A2:Voltage across LC = %5.2f V') %(a2))
            Fit4.config(text = _('Phase Shift = %5.1f deg') %(pd01_fit*180./math.pi))
            #Fit5.config(text = _('arc tan(Vx/Vr)= %5.1f deg') %(pd01*180./math.pi))
            vp.line((0,0),(0, a1), col =1)
            rx = a0 * math.sin(pd01_fit)
            ry = a0 * math.cos(pd01_fit) 
            vp.line((0,rx),(0,ry))
            vp.line((0,sign*a2),(0, 0), col=2)
        else:
            msg(_('Curve Fitfing failed. Try changing X scale'))
    except:
        msg(_('Capture Error. Check input voltage levels.'),'red')
        CMERR = True
    root.after(10,update)

def set_vertical(w):
    global delay, NP, NC, VPERDIV
    divs = [5.0, 1.0, 0.5, 0.2]
    VPERDIV = divs[int(Vpd.get())]
    g.setWorld(0,-5*VPERDIV, NP * delay * 0.001, 5*VPERDIV,_('mS'),_('V'))

def save_data():        
    global data
    fn = Fname.get()
    p.save(data,fn)
    msg(_('Traces saved to %s') %fn)

def xmgrace():
    global data
    if p.grace(data) == False:
        msg(_('Could not find Xmgrace or Pygrace. Install them'),'red')
    else:
        msg(_('Traces send to Xmgrace'))


def calc():
    try:
        f = float(Freq.get())
        C = float(Cap.get())
        L = float(Ind.get())
        R = float(Res.get())
        Xl = 2*math.pi*f*L*1.e-3
        if C != 0:
            Xc = 1./(2*math.pi*f*C*1.e-6)
        else:
            Xc = 0.0
        dphi = math.atan( (Xc-Xl)/R)*180./math.pi
        s = _('XC = %5.1f   XL = %5.1f\nDphi = %5.1f degree') %(Xc, Xl, dphi)
        Calc2.config(text = s, fg='blue')
    except:
        Calc2.config(text = _('Wrong Input'), fg='red')

#===================== main program starts here ===========================
p = eyes.open()
if p == None: sys.exit()

p.set_trigger(2048)
root = Tk()    
f = Frame(root)
f.pack(side=LEFT)
g = eyeplot.graph(f, width=WIDTH, height=HEIGHT)
# make plot objects using draw.disp
g.setWorld(0, -5, 20, 5,_('mS'),_('V'))

f1 = Frame(f)
f1.pack(side=TOP,  fill = BOTH, expand = 1)
Label(f1,text = _('mSec/div')).pack(side=LEFT, anchor = SW)
# Sliders for Adjusting Axes
timebase = Scale(f1,command = set_timebase, orient=HORIZONTAL,
                 length=LPWIDTH, showvalue=False,
                 from_ = 0, to=9, resolution=1)
timebase.pack(side=LEFT, anchor = SW)
timebase.set(3)

Label(f1,text = _('Volt/div')).pack(side=LEFT, anchor = SW)
Vpd = Scale(f1,command = set_vertical, orient=HORIZONTAL,
            length=LPWIDTH, showvalue=False,
            from_ = 0, to=3, resolution=1)
Vpd.pack(side=LEFT, anchor = SW)
Vpd.set(1)

Save = Button(f1,text=_('Save Traces to'), command = save_data)
Save.pack(side=LEFT, anchor=N)
Fname = Entry(f1, width=8)
Fname.pack(side=LEFT)
Fname.insert(0,'cro.txt')

Save = Button(f1,text=_('XmGrace'), command = xmgrace)
Save.pack(side=LEFT, anchor=N)
Quit = Button(f1,text=_('QUIT'), command = sys.exit)
Quit.pack(side=LEFT, anchor=N)

mf = Frame(f)
mf.pack(side=TOP,  fill = BOTH, expand = 1)
msgwin = Label(mf,text = _('Connect SINE to A1, R from A2 to GND. Inductor and/or Capacitor from A1 to A2.'), fg = 'blue')
msgwin.pack(side=LEFT)#, anchor = SW)

#============ Right Side panel ===============================
RFWIDTH = 150
rf = Frame(root, width = RFWIDTH, height = HEIGHT)
rf.pack(side=LEFT,  fill = BOTH, expand = 1)
vp = eyeplot.graph(rf, width=RFWIDTH, height=RFWIDTH, labels=True)
# make plot objects using draw.disp
vp.setWorld(-5, -5, 5, 5,'','')
Label(rf,text = _('Phasor Plot')).pack(side=TOP)

Fit0 = Label(rf, text = '', fg='magenta')
Fit0.pack(side = TOP, anchor = W)
Fit1 = Label(rf, text = '', fg='black')
Fit1.pack(side = TOP, anchor = W)
Fit2 = Label(rf, text = '', fg='red')
Fit2.pack(side = TOP, anchor = W)
Fit3 = Label(rf, text = '', fg='blue')
Fit3.pack(side = TOP, anchor = W)
Fit4 = Label(rf, text = '', fg='magenta')
Fit4.pack(side = TOP, anchor = W)
Fit5 = Label(rf, text = '', fg='magenta')
Fit5.pack(side = TOP, anchor = W)

Label(rf,text = _('Calculator')).pack(side=TOP, anchor=N)
f = Frame(rf)
f.pack(side=TOP, anchor = W)
Label(f,text = _('Freq=')).pack(side=LEFT)
Freq = Entry(f, width = 4)
Freq.pack(side=LEFT, anchor = N)
Freq.insert(0,'150')
Label(f,text = _('Hz R=')).pack(side=LEFT)
Res = Entry(f, width = 4)
Res.pack(side=LEFT, anchor = N)
Res.insert(0,'1000')
Label(f,text = _('Ohm')).pack(side=LEFT)

f = Frame(rf)
f.pack(side=TOP, anchor = W)
Label(f,text = _('C=')).pack(side=LEFT)
Cap = Entry(f, width = 4)
Cap.pack(side=LEFT, anchor = N)
Cap.insert(0,'1')
Label(f,text = _('uF. L=')).pack(side=LEFT)
Ind = Entry(f, width = 4)
Ind.pack(side=LEFT, anchor = N)
Ind.insert(0,'100')
Label(f,text = _('mH')).pack(side=LEFT)

Calc = Button(rf,text=_('Calculate XL, XC and Angle'),command = calc)
Calc.pack(side=TOP)
Calc2 = Label(rf,text = '')
Calc2.pack(side=TOP)
#---------------

eyeplot.pop_image('pics/ac-circuits.png', _('Study of AC Circuit'))
root.title(_('Study of AC Circuits'))
root.after(10,update)
root.mainloop()

# Local Variables:
# python-indent: 4
# End:



'''
expEYES Explorer program
Author  : Ajith Kumar B.P, bpajith@gmail.com
License : GNU GPL version 3
'''

import gettext
gettext.bindtextdomain("expeyes")
gettext.textdomain('expeyes')
_ = gettext.gettext

from Tkinter import *
import Image, ImageTk, tkFont, os, sys, commands, math
import expeyes.eyes as eyes, expeyes.eyeplot as eyeplot

try:		
	import expeyes.eyemath as eyemath		# Will fail if scipy is not installed
	EYEMATH = True
except:
	EYEMATH = False


WIDTH  = 555.0 * 0.8
HEIGHT = 677.0 * 0.8
BORDER =  0 #HEIGHT/8
TIMER  = 50
picture = 'pics/eyes.png'
pgreen = '#d1e244'


help = [
_('For help, click on the Terminal Boxes(1 to 32).\nLIZ : Lissajous figure.\n')+\
_('FT : Fourier Transform power spectrum.\nXM : Xmgrace 2D plotting program\n')+\
_('XmGrace is NOT available under MSWindows'),
_('1.Software can read the voltage input level, LOW ( < .8V) or HIGH (>2V).\n') +\
_('If a square wave input is given, click on the Buttons for measuring frequency / duty cycle'),
_('2. Can sense input level'),
_('3. Digital Output.  Can be set to 0 or 5 volts.\nUse the Checkbutton to change the Level'),
_('4. Digital Output.  Can be set to 0 or 5 volts.\nUse the Checkbutton to change the Level'),
_('5. Ground (zero volts)'),
_('6. SQR1: Generates Square Wave. Voltage swings between 0 and 5V. Frequency is programmable from ')+\
_('Hz to1 MHz. All intermediate values of frequency are not possible.'),
_('7. SQR2: Generates Square Wave. The frequency range is controlled by software and fine adjustment ')+\
_('is done by an external 22 kOhm variable resistor. Frequency range is from 0.7 Hz to 90 kHz.'),
_('8. 22 kOhm resistor used for frequency adjustment of SQR2.'),
_('9. 22 kOhm resistor used for frequency adjustment of SQR2.'),
_('10. Programmable Pulse. Frequency is 488.3 Hz. Duty cycle from 0 to 100%% in 255 steps.'),
_('11. Ground'),
_('12. Output of Inverting Amplifier with a gain of 47. (Input at 14)'),
_('13. Output of Inverting Amplifier with a gain of 47. (Input at 15)'),
_('14. Input of Inverting Amplifier with a gain of 47. (Output at 12)'),
_('15. Input of Inverting Amplifier with a gain of 47. (Output at 13). Also acts as a Frequency counter, ')+\
_('for a bipolar a signal (amplitude from 100 mV to 5V). If the signal is unipolar feed it ')+\
_('via a series capacitor'),
_('16. Ground'),
_('17. Input of Inverting Amplifier. Default Gain=100. The gain can be reduced by a series resistor at the input. ')+\
_('The gain will be given by G = 10000/(100+R), where R is the value of the external series resistor.'),
_('18. Output of the Inverting Amplifier (Input 17)'),
_('19. Ground'),
_('20. Gain control resistor for Non-Inverting amplifier, from 20 to Ground. Gain = 1 + 10000/Rg.'),
_('21. Input of Non-Inverting Amplifier (Output 22)'),
_('22. Output of Non-Inverting Amplifier(Input 21)'),
_('23. Sensor Input. Connect Photo transistor collector here and emitter to Ground.'),
_('24. Voltage measurement terminal. Input must be in the 0 to 5V range.'),
_('25. Voltage measurement terminal. Input must be in the -5V to 5V range.'),
_('26. Voltage measurement terminal. Input must be in the -5V to 5V range.'),
_('27. Ground'),
_('28. Programmable constant current source. 0.05 to 2 milli ampere range. The load resistor ')+\
_('should be chosen to make the product of I and R less than 2 volts.'),
_('29. Output of 30 through a 1kOhm resistor. Used for doing diode I-V characteristic.'),
_('30. Programmable voltage between -5V to +5V.'),
_('31. Programmable voltage between 0 to +5V.'),
_('32. Sine wave output. Frequency around 90 Hz. Voltage swings between -4V to +4V.')
]
class eyePanel:
	NSIG = 1 + 32						# zeroth element is unused
	tw = [None] * NSIG				    # List to store the widget variables created on the Panel
	LE = [ 6,  7, 10]					# Entry widget on left side
	LL = [ 1, 2, 8, 15]					# Lebel widgets on left side
	RE = [28, 30, 31]					# Entry widget on left side
	RL = [22,23,24,25,26,27]			# Lebel widgets on left side
	doutval = [None] * 2				# IntVar() of CheckButton widgets
	doutCB  = [None] * 2				# Checkbutton widgets
	NOSQR2 = True						# SQR2 is not set
	NOSF = True							# No frequency on SENSOR input
	NOAF = True							# No frequency on Amplifier input, T15
	NODF = True							# No frequency on Digital input 0
	OUTMASK = 0							# Digital outputs to LOW
	trace = None
	poped = False

	def pop_expt_menu(self,event):
		self.poped = True
		menu.post(event.x_root, event.y_root)

	limits = {3:(0,1), 4:(0,1), 6:(0,100000.), 7:(-1,100000.), 10:(0.,100.0), \
					28:(0.020, 3.0), 29:(-5.,5.), 30:(-5.,5.), 31:(0.,5.) }
	def get_fieldvalue(self,i):
		try:
			s = self.tw[i].get()
			val = float(s)
			if self.limits[i][0] <= val <= self.limits[i][1]:
				return val
		except:
			pass

	def __init__(self, parent, handle, width=WIDTH, height = HEIGHT):
		self.eye = handle 
		self.parent = parent
		self.width = width
		self.height = height
		self.border = BORDER				# Top and bottom self.border
		self.fw = float(width)/12.7			# field width
		self.fh = float(height - 2 * self.border)/16	# field height
		im = Image.open(eyeplot.abs_path()+ picture)
		im = im.resize((int(width),int(height)))
		self.image = ImageTk.PhotoImage(im)
		self.panel = Canvas(parent, width = width, height = height)
		self.panel.create_image(0,0,image = self.image, anchor = NW)

		self.popup = Button(text = _('EXPERIMENTS'), bg=pgreen)
		self.popup.bind("<ButtonRelease-1>", self.pop_expt_menu)
		self.panel.create_window(width/2, height-20, window = self.popup, anchor = CENTER)
		
		self.panel.bind("<ButtonRelease-1>", self.clicked)
		self.panel.bind("<ButtonRelease-3>", self.pop_expt_menu)
		self.panel.pack(side=TOP, anchor=SW)

		for i in self.LE:				# Text Entry Fields on left
			x,y = self.xyfromi(i)
			self.tw[i] = Entry(width = 8, bg = 'white', fg='blue')
			self.tw[i].bind("<Return>", self.process)
			self.tw[i].bind("<KP_Enter>", self.process)
			self.panel.create_window(3.5*self.fw, y, window = self.tw[i], anchor = W)

		for i in self.LL:				# Label widgets on left
			x,y = self.xyfromi(i)
			self.tw[i] = Label(width = 8, bg = pgreen, fg='blue', bd=1)
			self.tw[i].bind("<Return>", self.process)
			self.tw[i].bind("<KP_Enter>", self.process)
			if i == 8: y -= 12
			self.panel.create_window(3.5*self.fw, y, window = self.tw[i], anchor = W)

		for i in self.RE:				# Text Entry Fields on right
			x,y = self.xyfromi(i)
			self.tw[i] = Entry(width = 8, bg = 'white', fg='blue')
			self.tw[i].bind("<Return>", self.process)
			self.tw[i].bind("<KP_Enter>", self.process)
			self.panel.create_window(width-3*self.fw, y, window = self.tw[i], anchor = E)

		for i in self.RL:				# Text Entry Fields on right
			x,y = self.xyfromi(i)
			self.tw[i] = Label(width = 8, bg = pgreen, fg='blue', bd=1)
			self.tw[i].bind("<Return>", self.process)
			self.tw[i].bind("<KP_Enter>", self.process)
			if i == 27 or i == 22: y -= 12
			self.panel.create_window(width-3*self.fw, y, window = self.tw[i], anchor = E)

		for i in range(2):
			x,y = self.xyfromi(i+3)
			self.doutval[i] = IntVar()
			self.doutCB[i] = Checkbutton(bg = 'red', variable = self.doutval[i], \
					command = lambda i=i : self.checked(i))
			self.panel.create_window(3.5*self.fw, y, window = self.doutCB[i], anchor = W)
			self.doutCB[i].config(text=_('LO'),bg='gray')

			x,y = self.xyfromi(28)
			self.panel.create_line([width-self.fw, y, width-3*self.fw, y+self.fh/2], fill= pgreen)
			x,y = self.xyfromi(23)
			self.panel.create_line([width-self.fw, y, width-3*self.fw, y+self.fh/2], fill= pgreen)
			x,y = self.xyfromi(7)
			self.panel.create_line([self.fw, y, 3.5*self.fw, y+self.fh/2], fill= pgreen)


		x,y = self.xyfromi(1)
		self.FRB = Button(bg = 'gray', text =_('F'), padx=0, pady=0,	command = self.freq_id0)
		self.panel.create_window(3.0*self.fw, y, window = self.FRB, anchor = W)
		self.DCB = Button(bg = 'gray', text ='%', padx=0, pady=0,	command = self.duty_cycle)
		self.panel.create_window(5.5*self.fw, y, window = self.DCB, anchor = W)
		x,y = self.xyfromi(15)
		self.FRB = Button(bg = 'gray', text =_('F'), padx=0, pady=0,	command = self.freq_ampin)
		self.panel.create_window(3.0*self.fw, y, window = self.FRB, anchor = W)
		x,y = self.xyfromi(22)
		self.FRB = Button(bg = 'gray', text =_('F'), padx=0, pady=0,	command = self.freq_adc5)
		self.panel.create_window(width-6.2*self.fw, y, window = self.FRB, anchor = SW)
		self.looping = True


	def freq_adc5(self):
		fr = self.eye.sensor_frequency()
		if fr < 0:
			self.labset(22, _('0 Hz'))
			self.NOSF = True
		else:
			self.labset(22, '%5.2f Hz'%(fr))
			self.NOSF = False

	def freq_ampin(self):
		fr = self.eye.ampin_frequency()
		if fr < 0:
			self.labset(15, _('0 Hz'))
			self.NOAF = True
		else:
			self.labset(15, '%5.2f Hz'%(fr))
			self.NOAF = False

	def freq_id0(self):
		fr = self.eye.digin_frequency(0)
		if fr < 0:
			self.labset(1, _('0 Hz'))
		else:
			self.labset(1, '%5.2f Hz'%fr)

	def ifromxy(self,e):				# Calculates the Index from the xy coordinates
		#print e.x, e.x_root, e.y, e.y_root
		if self.border < e.y < self.height-self.border and (e.x < 2*self.fw or e.x > self.width-2*self.fw):
			if e.x < self.fw:
				return 1, int(float(e.y-self.border)/self.fh)+1
			elif e.x < 2*self.fw:
				return 2, int(float(e.y-self.border)/self.fh)+1
			elif e.x > self.width - self.fw:
				return 1, 31 - int(float(e.y-self.border)/self.fh)+1
			elif e.x > self.width - 2*self.fw:
				return 2, 31 - int(float(e.y-self.border)/self.fh)+1
		return 0,0	# Implies Invalid Field


	def xyfromi(self,i):		# Calculates the xy coordinates for placing widgets
		if i <= 16:
			return 1, self.border + (i-1)*self.fh + self.fh/2
		elif i <= 32:
			return self.width - self.fw, self.height - (i-16)*self.fh + self.fh/2 - self.border

	def save(self):
		self.eye.save(self.trace,'explore.dat')
		showhelp(_('Traces saved to explore.dat'))

	def xmgrace(self):
		if self.eye.grace(self.trace) == False:
			showhelp(_('Could not find Xmgrace or Pygrace. Install them'),'red')


	def do_fft(self):
		global delay, NP, NC, EYEMATH
		if EYEMATH == False:
			showhelp(_('Could not find scipy package. Install it'),'red')
			return
		if self.trace == None: return
		transform = []
		for xy in self.trace:
			fr,tr = eyemath.fft(xy[1], delay * NC * 0.001)
			transform.append([fr,tr])
		self.eye.save(transform, 'exploreFFT.dat')
		self.eye.grace(transform, _('freq'), _('power'))
		showhelp(_('Fourier transform Saved to exploreFFT.dat.'))

	def labset(self,i,s):
		self.tw[i].config(text=s)

	def twset(self,i,s):
		self.tw[i].delete(0,END)
		self.tw[i].insert(0,s)

	def process(self,e):							# Enter key in any of the Text Entry Fields
		for i in range(self.NSIG):
			if self.tw[i] == e.widget:				# Look for the widget where Enter is pressed			
				fld = i
				break
		msg = ''
		val = self.get_fieldvalue(fld)					# Get the value entered by the user
		if val == None:	
			return
		elif fld == 6:					# Set SQR1
			freq = self.eye.set_sqr1(val)
			self.twset(fld,'%5.1f'%freq)
		elif fld == 7:					# Set SQR2
			self.eye.set_sqr2(val)
			freq = self.eye.get_sqr2()
			if freq > 0:
				self.labset(8,'%5.1f Hz'%freq)
				self.NOSQR2 = False
			else:
				self.labset(8, _('0 Hz'))
				self.NOSQR2 = True
		elif fld == 10:					# Set Pulse duty cycle
			ds = self.eye.set_pulse(val)
			self.twset(fld,'%5.1f'%ds)
		elif fld == 28:					# Set Current
			self.eye.set_current(val)
			self.twset(fld,'%5.3f'%val)
		elif fld == 30:
			self.eye.set_voltage(0,val)
			self.twset(i,'%5.3f'%val)
		elif fld == 31:
			self.eye.set_voltage(1,val)
			self.twset(fld, '%5.3f'%val)

	def clicked(self,e):
		if self.poped == True:		# Remove poped menu by cicking else where
			menu.unpost()
			self.poped = False
		a,i = self.ifromxy(e)
		if a == 1:
			showhelp(help[i])
		#print e.x, e.y, a, i


	def duty_cycle(self):
		hi = self.eye.r2ftime(0,0)
		if hi > 0:
			lo = self.eye.f2rtime(0,0)
			ds = 100*hi/(hi+lo)
			self.labset(1, '%5.2f %%'%(ds))
		else:
			self.labset(1,_('0 Hz'))

	def checked(self, i):		# Clicked Checkbutton
		val  = self.doutval[i].get()
		if val == 0:
			self.OUTMASK &= ~(1 << i)
			self.doutCB[i].config(text=_('LO'),bg='gray')
		elif val == 1:
			self.OUTMASK |= (1 << i)
			self.doutCB[i].config(text=_('HI'), bg='green')
		self.eye.write_outputs(self.OUTMASK & 3)


	def routine_work(self):
		global NP, delay, chanmask, measure, lissa, EYEMATH
		s = ''
		self.trace = []
								# In the final stage, move this to a Try block.
		if lissa == True:
			t,v,tt,vv = self.eye.capture01(NP,delay)
			g.delete_lines()
			g.setWorld(-5,-5,5,5,_('mS'),_('V'))
			g.line(v,vv)
			self.trace.append([v,vv])
		elif chanmask == 1 or chanmask == 2:				# Waveform display code 
			t, v = self.eye.capture(chanmask-1,NP,delay)
			g.delete_lines()
			g.line(t,v,chanmask-1)
			self.trace.append([t,v])
		elif chanmask == 3:
			t,v,tt,vv = self.eye.capture01(NP,delay)
			g.delete_lines()
			g.line(t,v)
			g.line(tt,vv,1)
			self.trace.append([t,v])
			self.trace.append([tt,vv])
		if measure == 1 and EYEMATH == False:
			showhelp(_('python-scipy not installed. Required for data fitting'),'red')
		if measure == 1 and lissa == False and EYEMATH == True:		# Curve Fitting
			if chanmask == 1 or chanmask == 2:			
				fa = eyemath.fit_sine(t, v)
				if fa != None:
					#g.line(t,fa[0], 8)
					rms = self.eye.rms(v)
					f0 = fa[1][1] * 1000
					s = _('CH%d %5.2f V , F= %5.2f Hz') %(chanmask>>1, rms, f0)
				else:
					s = _('CH%d nosig ')%(chanmask>>1)

			elif chanmask == 3:	
				fa = eyemath.fit_sine(t,v)
				if fa != None:
					#g.line(t,fa[0],8)
					rms = self.eye.rms(v)
					f0 = fa[1][1]*1000
					ph0 = fa[1][2]
					s += _('CH0 : %5.2f V , %5.2f Hz ') %(rms, f0)
				else:
					s += _('CH0: no signal ')
				fb = eyemath.fit_sine(tt,vv)
				if fb != None:
					#g.line(tt,fb[0],8)
					rms = self.eye.rms(vv)
					f1 = fb[1][1]*1000
					ph1 = fb[1][2]
					s = s + _('| CH1 %5.2f V , %5.2f Hz') %(rms, f1)
					if fa != None and abs(f0-f1) < f0*0.1:
						s = s + _(' | dphi= %5.1f')%( (ph1-ph0)*180.0/math.pi)
				else:
					s += _('| CH1:no signal ')
		msgwin.config(text=s)			# CRO part over	

		v = self.eye.get_voltage(6)			# CS voltage
		self.labset(27, '%5.3f V'%v)					
		v = self.eye.get_voltage(0)			# A0
		self.labset(26, '%5.3f V'%v)
		v = self.eye.get_voltage(1)			# A1
		self.labset(25, '%5.3f V'%v)
		v = self.eye.get_voltage(2)			# A2
		self.labset(24, '%5.3f V'%v)
		v = self.eye.get_voltage(4)			# SENSOR
		self.labset(23, '%5.3f V'%v)

		res = self.eye.read_inputs()		# Set the color based on Input Levels
		if res & 1:								# ID0
			self.tw[1].config(bg = pgreen)				
		else:
			self.tw[1].config(bg = 'gray')		
		if res & 2:								# ID1
			self.tw[2].config(bg = pgreen)		
		else:
			self.tw[2].config(bg = 'gray')	
		if res & 4:								# T15 input
			self.tw[15].config(bg = pgreen)		
		else:
			self.tw[15].config(bg = 'gray')	
		if res & 8:								# Sensor Input
			self.tw[22].config(bg = pgreen)		
		else:
			self.tw[22].config(bg = 'gray')	

		if self.NOSQR2 == False:
			freq = self.eye.get_sqr2()
			if freq > 0:
				self.labset(8,'%5.1f Hz'%freq)
			else:
				self.labset(8, _('0 Hz'))
				self.NOSQR2 = True

		if self.NOSF == False:
			freq = self.eye.sensor_frequency()
			if freq > 0:
				self.labset(22,'%5.1f Hz'%freq)
			else:
				self.labset(22, _('0 Hz'))
				self.NOSF = True

	def update(self):
		try:
			self.routine_work()
		except:
			showhelp(_('Transaction Error.'),'red')
		root.after(TIMER, self.update)
#----------------------------------------Panel class ends ------------------------------------------

VPERDIV = 1.0		# Volts per division, vertical scale
delay = 10			# Time interval between samples
NP = 100			# Number of samples
NC = 1				# Number of channels
chanmask = 1		# 01, 10 or 11 binary
measure  = 0
lissa = 0

def showhelp(s,col='black'):
	helpwin.delete(1.0, END)
	helpwin.config(fg=col)
	helpwin.insert(END, s)

def set_vertical(w):
	global delay, NP, NC, VPERDIV
	divs = [1.1, 1.0, 0.5, 0.2, 0.1, 0.05, 0.02]
	VPERDIV = divs[int(vpd.get())]
	g.setWorld(0,-5*VPERDIV, NP * delay * 0.001, 5*VPERDIV,_('mS'),_('V'))

def set_timebase(w):
	global delay, NP, NC, VPERDIV
	divs = [0.050, 0.100, 0.200, 0.500, 1.0, 2.0, 5.0, 10.0, 20.0, 50.0]
	msperdiv = divs[int(timebase.get())]
	totalusec = int(msperdiv * 1000 * 10)
	NP = 200								# Assume 100 samples to start with
	delay = int(totalusec/100)				# Calculate delay
	if delay < 10:
		sf = 10/delay
		delay = 10
		NP = NP/sf * NC
	elif delay > 1000:
		sf = delay/1000
		delay = 1000
		NP = NP * sf / NC
	g.setWorld(0,-5*VPERDIV, NP * delay * 0.001, 5*VPERDIV,_('mS'),_('V'))
	#print NP, NC, delay

def select_chan():
	global chanmask, measure, NC
	chanmask = CH0.get() | (CH1.get() << 1)
	measure = FIT.get()
	if chanmask == 3: 
		NC =2
	else:
		NC = 1

def lissa_mode():
	global lissa,delay, NP, NC, VPERDIV
	lissa = LIZ.get()
	if lissa == 1:
		lissa = True
	else:
		g.setWorld(0,-5*VPERDIV, NP * delay * 0.001, 5*VPERDIV,_('mS'),_('V'))	# Restore old scale
		lissa = False

#-----------------------------main program starts here-----------------------------
for k in range(10):		# Test the hardware availability by by running another program.
	stat,out = commands.getstatusoutput('python '+ eyeplot.abs_path() + 'hwtest.py')
	if stat == 0:
		break

pe = eyes.open()			# Try several times to make a connection
root = Tk()

left = Frame(root)			# Divide root window into Left and Right
left.pack(side=LEFT, anchor = S)
right = Frame(root)
right.pack(side = LEFT, anchor = S, fill = Y),

w=eyePanel(left, pe, WIDTH, HEIGHT)		# Panel photograph to the Left Panel
g = eyeplot.graph(right, WIDTH*1.05, HEIGHT*2./3,color = 'white', labels=False)  # Plot window 
g.setWorld(0,-5*VPERDIV, NP * delay * 0.001, 5*VPERDIV,_('mS'),_('V'))

cf = Frame(right)						# Command Frame, inside the right frame, below plot window
cf.pack(side=TOP, anchor = NW)
l = Label(cf, text=_('mS/div'))
l.pack(side=LEFT, anchor = SW )
timebase = Scale(cf,command = set_timebase, orient=HORIZONTAL, length=50, showvalue=False,\
	from_ = 0, to=9, resolution=1)
timebase.pack(side=LEFT, anchor = SW)
timebase.set(2)

'''
l = Label(text = _('Volt/div'))
vpd = Scale(cf,command = set_vertical, orient=HORIZONTAL, length=50, showvalue=False,\
	from_ = 0, to=2, resolution=1)
vpd.pack(side=LEFT, anchor = SW)
vpd.set(1)
'''

CH0 = IntVar()
cb0 = Checkbutton(cf,text ='A0', command=select_chan, variable=CH0, fg = 'black')
cb0.pack(side=LEFT, anchor = SW)
CH0.set(1)
CH1 = IntVar()
cb1 = Checkbutton(cf,text ='A1', command=select_chan, variable=CH1, fg = 'red')
cb1.pack(side=LEFT, anchor = SW)
CH1.set(0)
LIZ = IntVar()
liz = Checkbutton(cf,text =_('LIZ'), command=lissa_mode, variable=LIZ, fg = 'black')
liz.pack(side=LEFT, anchor = SW)
LIZ.set(0)

FIT = IntVar()
b=Checkbutton(cf,text=_('FIT'), command = select_chan, variable=FIT, fg= 'black')
b.pack(side=LEFT, anchor = SW)
b = Button(cf,text =_('Save'), command=w.save)
b.pack(side=LEFT, anchor = SW)
b = Button(cf,text =_('FT'), command=w.do_fft)
b.pack(side=LEFT, anchor = SW)

b = Button(cf,text =_('XM'), command=w.xmgrace)
b.pack(side=LEFT, anchor = SW)
b = Button(cf,text =_('QUIT'), command=sys.exit)
b.pack(side=LEFT, anchor = SW)

mf = Frame(right)				# Message Frame below command frame.
mf.pack(side=TOP, anchor = SW)
msgwin = Label(mf,text = _('Messages'), fg = 'blue')
msgwin.pack(side=LEFT, anchor = SW)

f = Frame(right, bg= 'white')
f.pack(side = TOP)
font = tkFont.Font(family = 'helvetica', size = 12)
scrollbar = Scrollbar(f)
scrollbar.pack(side=RIGHT, fill=Y)
f.pack(side = TOP, fill = BOTH, expand = 1)
helpwin = Text(f, width = 45, height = 5, font = font, fg = 'black', bg = 'white', spacing2 = 0,\
		wrap=WORD, yscrollcommand=scrollbar.set)
helpwin.pack(side = TOP, fill = BOTH, expand = 1)
scrollbar.config(command=helpwin.yview)
showhelp(help[0])

#------------------------popup menu ---------------------------
expts = [ 
[_('Resistor IV'),'resistor_iv'],
[_('RC Circuit'),'RCcircuit'],
[_('RL Circuit'),'RLcircuit'],
[_('RLC Discharge'),'RLCdischarge'],
[_('EM Induction'),'induction'],
[_('Diode IV'),'diode_iv'],
[_('LED IV'),'LED_iv'],
[_('Transistor CE'),'transistor'],
[_('Frequency Response'),'freq-response'],
[_('Velocity of Sound') , 'velocity-sound'],
[_('Interference of Sound') , 'interference-sound'],
[_('Photo-Transistor CE'),'phototransistor'],
[_('Rod Pendulum') , 'rodpend'],
[_('Gravity TOF'), 'gravity_tof'],
[_('Pendulum Wavefrorm'),'pendulum'],
[_('40 kHz Piezo TOF'),'usound_tof'],
[_('PT100 Sensor'), 'pt100'],
[_('Temp Comptroller'), 'temp-controller'],
[_('Data Logger'), 'logger'],
[_('CRO'),'cro'],
[_('AM and FM'), 'amfm'],
[_('Music'),'janagana'],
[_('Calibrate'),'calibrate']
 ]


def run_expt(expt):
	global w
	w.eye.fd.close()	# Close hardware port
	if os.name == 'nt':		# For windows OS
		cmd = sys.executable + ' ' + eyeplot.abs_path() + expt+'.py'
		os.system(cmd)
		showhelp(_('Finished ') + expt)
	else:
		#print abs_path() + expt+'.py'
		stat,out = commands.getstatusoutput('python '+ eyeplot.abs_path() + expt+'.py')
		if stat != 0:
			showhelp(out)
		else:
			showhelp(_('Finished "')+expt+'.py"')
	w.eye = eyes.open()	# Open hardware port again
	w.eye.disable_actions()

menu = Menu(w.panel, tearoff=0)
for k in range(len(expts)):
	text = expts[k][0]
	cmd = expts[k][1]
	#print text, cmd
	menu.add_command(label=text, background= 'ivory', command = lambda expt=cmd :run_expt(expt))

# Check Hardware
if pe == None:	
	root.title(_('EYES Hardware NOT found.'))
	showhelp(_('EYES Hardware Not Found.\nRe-Connect USB cable and restart the program.'), 'red')
	root.mainloop()
	sys.exit()
else:
	root.title(_('EYES Hardware found on ') + str(pe.device))
	pe.write_outputs(0)
	pe.disable_actions()
	pe.loadall_calib()
	root.after(TIMER,w.update)
root.mainloop()
  



'''
expEYES program
Author  : Ajith Kumar B.P, bpajith@gmail.com
License : GNU GPL version 3
'''
import gettext
gettext.bindtextdomain("expeyes")
gettext.textdomain('expeyes')
_ = gettext.gettext

import time, math, sys
if sys.version_info.major==3:
        from tkinter import *
else:
        from Tkinter import *

sys.path=[".."] + sys.path

import expeyes.eyesj as eyes
import expeyes.eyeplot as eyeplot
import expeyes.eyemath as eyemath

WIDTH  = 600   # width of drawing canvas
HEIGHT = 400   # height    

class Pend:
	tv = [ [], [] ]			# Lists for Readings
	TIMER = 5			# Time interval between reads
	IDLE_TIMER = 100                # time interval between idle reads
	idle_threshold = 3              # 3V are necessary to autostart
	MINY = -5			# Voltage range
	MAXY = 5
	running = False
	MAXTIME = 10

	def __init__(self):
		"""
		The constructor
		"""
		self.reset_idle()
		root.after(self.IDLE_TIMER, self.idle)

	def fit_curve(self):
		fa = eyemath.fit_dsine(self.tv[0], self.tv[1], mode="Hz")
		if fa != None:
			pa = fa[1]
			g.line(self.tv[0], fa[0],1)
			self.msg(_('Angular velocity = %5.2f rad/sec. Damping Factor = %5.3f')%(pa[1], pa[4]))
		else:
			self.msg(_('Failed to fit data'))

	def xmgrace(self):
		if self.running == True:
			return
		p.grace([self.tv])

	def reset_idle(self):
		"""
		resets an internal list of 20 samples
		"""
		self.idle_values=[0.0]*20
		return

	def idle(self):
		"""
		callback used when no other method is in action.
		stores internally 20 previous samples of voltage measured
		recently, every tenth of second.
		When this list contains values with an amplitude greater
		than a given threshold, creates an "autolaunch".
		"""
		if self.running:
			return
		t,v = p.get_voltage_time(1)
		del self.idle_values[0]
		self.idle_values.append(v)
		amplitude=max(self.idle_values) - min(self.idle_values)
		if amplitude > self.idle_threshold:
			self.start()
		root.after(self.IDLE_TIMER, self.idle)
		return
		
	def start(self):
		self.running = True
		self.reset_idle()
		self.index = 0
		self.tv = [ [], [] ]
		try:
			self.MAXTIME = int(DURATION.get())
			g.setWorld(0, self.MINY, self.MAXTIME, self.MAXY,_('Time'),_('Volt'))
			Dur.config(state=DISABLED)
			self.msg(_('Starting the Measurements'))
			root.after(self.TIMER, self.update)
		except:
			self.msg(_('Failed to Start'))

	def stop(self):
		self.running = False
		Dur.config(state=NORMAL)
		self.msg(_('User Stopped the measurements'))
		root.after(self.IDLE_TIMER, self.idle)

	def update(self):
		if self.running == False:
			return
		t,v = p.get_voltage_time(1)  # Read A2
		if len(self.tv[0]) == 0:
			self.start_time = t
			elapsed = 0
		else:
			elapsed = t - self.start_time
		self.tv[0].append(elapsed)
		self.tv[1].append(v)
		if len(self.tv[0]) >= 2:
			g.delete_lines()
			g.line(self.tv[0], self.tv[1])
		if elapsed > self.MAXTIME:
			self.running = False
			Dur.config(state=NORMAL)
			self.msg(_('Completed the Measurements'))
			root.after(self.IDLE_TIMER, self.idle)
			return 
		root.after(self.TIMER, self.update)

	def save(self):
		try:
			fn = filename.get()
		except:
			fn = 'pendulum.dat'
		p.save([self.tv],fn)
		self.msg(_('Data saved to %s')%fn)

	def clear(self):
		if self.running == True:
			return
		self.tv = [ [], [] ]
		g.delete_lines()
		self.msg(_('Cleared Data and Trace'))

	def msg(self,s, col = 'blue'):
		msgwin.config(text=s, fg=col)

p = eyes.open()
p.disable_actions()
root = Tk()
Canvas(root, width = WIDTH, height = 5).pack(side=TOP)  # Some space at the top
g = eyeplot.graph(root, width=WIDTH, height=HEIGHT, bip=False)	# make plot objects using draw.disp
pen = Pend()

cf = Frame(root, width = WIDTH, height = 10)
cf.pack(side=TOP,  fill = BOTH, expand = 1)


b3 = Label(cf, text = _('Digitize for'))
b3.pack(side = LEFT, anchor = SW)
DURATION = StringVar()
Dur =Entry(cf, width=5, bg = 'white', textvariable = DURATION)
DURATION.set('15')
Dur.pack(side = LEFT, anchor = SW)
b3 = Label(cf, text = _('Seconds.'))
b3.pack(side = LEFT, anchor = SW)

cf = Frame(root, width = WIDTH, height = 10)
cf.pack(side=TOP,  fill = BOTH, expand = 1)
b1 = Button(cf, text = _('START'), command = pen.start)
b1.pack(side = LEFT, anchor = N)
b1 = Button(cf, text = _('STOP'), command = pen.stop)
b1.pack(side = LEFT, anchor = N)
b1 = Button(cf, text = _('FIT'), command = pen.fit_curve)
b1.pack(side = LEFT, anchor = N)
b4 = Button(cf, text = _('CLEAR'), command = pen.clear)
b4.pack(side = LEFT, anchor = N)
b1 = Button(cf, text = _('Xmgrace'), command = pen.xmgrace)
b1.pack(side = LEFT, anchor = N)
b3 = Button(cf, text = _('SAVE to'), command = pen.save)
b3.pack(side = LEFT, anchor = N)
filename = StringVar()
e1 =Entry(cf, width=15, bg = 'white', textvariable = filename)
filename.set('pendulum.dat')
e1.pack(side = LEFT)
b5 = Button(cf, text = _('QUIT'), command = sys.exit)
b5.pack(side = RIGHT, anchor = N)

mf = Frame(root, width = WIDTH, height = 10)
mf.pack(side=TOP)
msgwin = Label(mf,text=_('Message'), fg = 'blue')
msgwin.pack(side=LEFT, anchor = S, fill=BOTH, expand=1)


eyeplot.pop_image('pics/pend-wave.png', _('Pendulum Oscillations'))
root.title(_('Oscillations of Pendulum'))
root.mainloop()




# -*- coding: utf-8 -*-
### setup.py ###

from distutils.core import setup
setup (name='expeyes',
      version='1.0.0',
      description=u"a hardware & software framework for developing science experiments",
      author='Ajith Kumar B.P',
      author_email='bpajith@gmail.com',
      url='http://expeyes.in/',
      license='GPLv3',
      packages=['expeyes'],
      package_dir={'expeyes': "expeyes"},
)



# -*- coding: utf-8 -*-
import sys, re, codecs, datetime

dt=datetime.datetime.now()
data=[]

l=sys.stdin.readline().strip()
separators=re.compile(r"\s+")
names=separators.split(l)
while l:
	l=sys.stdin.readline().strip()
	if l:
		data.append(separators.split(l))
	
cols=max([len(l) for l in data])
rows=len(data)
t = newTable("Mes données", rows, cols)
t.setColNames(names)

row=1
for l in data:
	col=1
	for val in l:
		t.setCellData(col,row,val)
		col+=1
	row+=1

colTuple=tuple(names)
g1=plot(table("Mes données"), colTuple, Layer.LineSymbols)

l=g1.activeLayer()
l.setTitle(dt.strftime("%Y-%m-%d %H:%M:%S"))

